#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
# =================================================
import logging
import sys
import copy
import weakref

from chipwhisperer.common.utils import util
import functools

#TODO: pyqtgraph raises a general Exception(), so we can't actually test for importerror. Perhaps should manually check
#      for pyside first
class Parameterized(object):
    """
    Abstract class that implements basic functionality required by parameterized objects.
    All parameterized objects should have _name overriden in the class. The objects can also override this attribute
     with different names in the case of having two or more instances. The _description is optional.
    """
    _name = "None"
    _description = ""

    def getParams(self):
        """Return a parameter for the object. Creates it in the first time it is executed."""
        if not hasattr(self, "params"):
            self.params = Parameter(name=self.getName(), type='group')
            if self._description != "":
                self.params.addChildren(
                    [{'name': '', 'type': 'label', 'value': self.getDescription(), 'readonly': True}])
        return self.params

    def findParam(self, nameOrPath):
        """Return the paramenter child with the given name/path."""
        return self.getParams().getChild(nameOrPath)

    def getName(self):
        """Return the final name (instance name)."""
        return self._name

    @classmethod
    def getClassName(cls):
        """Return the original name (class name)."""
        return cls._name

    @classmethod
    def getDescription(cls):
        """Return the description of the parameterized class."""
        return cls._description

    def deleteParams(self):
        """Delete its parameters. Helps the GC in doing its work."""
        if hasattr(self, "params"):
            self.getParams().delete()
            del self.params


class Parameter(object):
    """
    Basic unity of data (like an attribute of a class) that can accessed in a more uniform way (like using search) and
    displayed in the screen with advanced widgets if QT+PyQtGraph is supported.
    Each parameter has a name, a type, a value, and several other properties that modify the behavior. The value can
    be saved internally to the Parameter or externally, using get/set methods. An action function is called every time
    the value of the parameter changes.

    Supported types:
    "group"               - A label with children
    "list"                - A value to be selected between a set of allowed values
    "label"               - Used in the description of Parameterized objects
    "str", "text"         - A smaller and a larger textbox
    "bool"                - A checkbox
    "action"              - A button
    "int", "float"        - A value to be selected between a min and max limit
    "range", "rangegraph" - An interval and an interval that can be selected using a graph widget (graphwidget option)
    "file"                - A string type with a file dialog button
    "filelist"            - Complex widged with a file list and mangement buttons to add/remove/edit/copy/set active.
     "color"              - Opens a color seletion dialog
     "menu"               - Hiden widget that inserts a new menu option

    Supported Attributes:
    "name", "type"        - Mandatory attributes
    "key"                 - A nickname to make search easier
    "values"              - Allowed set of values (list or dictionary type)
    "value"               - Value saved internally
    "set", "get"          - Value save externally
    "limits"              - Range of allowed values
    "step"                - Increment in SpinBoxes
    "linked"              - Refreshes (calls set(get)) the values in the given list with other parameters
    "default"             - Set the initial value (skips initialization when using set/get)
    "tip"                 - Description of the parameter
    "action"              - Calls the provided method with the current paramenter as argument when the value is changed
    "visible"             - Show/hides the parameter
    "children"            - Insert other parameters to be accessed within the current parameter hierarchy
    "readonly"            - Prevents the user of changing its value (it can be forced though)
    "help"                - Text displayed when clicking the help button
    "graphwidget"         - Reference to the graph widget when using parameters with type "rangegraph"
    'siPrefix', 'suffix'  - Adds prefix and sets the suffix text
    "psync"               - Disable reverse synchronization when calling the set method directly (no decorator needed)
    "addLoadSave"         - Adds load / save settings button to a group parameter
    ...

    Examples:
        self.getParams().addChildren([
            {'name':'Redraw after Each', 'type':'bool', 'value':False},
            {'name':'Trace Range', 'key':'tracerng', 'type':'range', 'limits':(0, 0), 'value':(0, 0)},
            {'name':'Point Range', 'key':'pointrng', 'type':'rangegraph', 'limits':(0, 0), 'value':(0, 0), 'graphwidget':self},
            {'name':'Redraw', 'type':'action', 'action':lambda _: self.plotInputTrace()},
            {'name':'Y-Offset', 'key':'yoffset', 'type':'float', 'step':1E-3, 'siPrefix': True, 'suffix': 'V'},
            {'name':'Update Mode', 'key':'updateMode', 'type':'list', 'values':{'Entire Table (Slow)':'all',
             'PGE Only (faster)':'pge'}, 'get':self.getUpdateMode, 'set':self.setUpdateMode},
            {'name':'Trace color', 'type':'color', 'value':"0F0", 'action':lambda p: self.setTraceColor(p.getValue())},
            {'name':'Draw Type', 'type':'list', 'key':'drawtype', 'values':['Fastest', 'Normal', 'Detailed'], 'value':'Normal',
                                 'help':"Draw Type Help"},
            {'name':'CW Firmware Preferences','tip':'Configure ChipWhisperer FW Paths', 'type':"menu",
             "action":lambda _:self.getFwLoaderConfigGUI.show()},
            #  "default" means set() is NOT called during init, otherwise set WILL be called to set the default value (read from get()). Can be useful when don't want to read hardware at start-up
            {'name':'SAD Threshold', 'type':'int', 'limits':(0, 100000), 'default':0, 'set':self.setThreshold, 'get':self.getThreshold}
            {'name':'Acquisition Settings', 'type':'group', 'children':[
                    {'name':'Number of Traces', 'type':'int', 'limits':(1, 1E9), 'get':self.getNumTraces, 'set':self.setNumTraces, 'linked':['Traces per Set']},
                    {'name':'Number of Sets', 'type':'int', 'limits':(1, 1E6), 'get':self.getNumTraceSets, 'set':self.setNumTraceSets, 'linked':['Traces per Set'], 'tip': 'Break acquisition into N sets, '
                     'which may cause data to be saved more frequently. The default capture driver requires that NTraces/NSets is small enough to avoid running out of system memory '
                     'as each segment is buffered into RAM before being written to disk.'},
                    {'name':'Traces per Set', 'type':'int', 'readonly':True, 'get':self.tracesPerSet},
            ]}
        ])
    """

    sigParametersChanged = util.Signal()
    registeredParameters = {}
    scriptingOutput = sys.stdout
    supportedTypes = ["group", "list", "label", "str", 'text', "bool", "action", "int", "float", "rangegraph", "file",
                      'filelist', "range", "color", "menu"]
    suppertedAttributes = {"name", "key", "type", "values", "value", "set", "get", "limits", "step", "linked",
                           "default", "tip", "action", "visible", "children", "readonly", "graphwidget"}
    usePyQtGraph = False

    def __init__(self, parent=None, ignoreChildren=False, **opts):
        self.sigValueChanged = util.Signal()
        self.sigLimitsChanged = util.Signal()
        self.sigOptionsChanged = util.Signal()
        self.sigChildAdded = util.Signal()
        self.sigChildRemoved = util.Signal()

        self.previousValue = None
        self.parent = parent
        self.invalid = False
        self.opts = {"visible": True}
        self.opts.update(opts)

        if 'name' not in self.opts or not isinstance(self.opts['name'], str):
            raise Exception("Parameter must have a name.")

        name = self.opts["name"]
        if 'type' not in self.opts or not isinstance(self.opts['type'], str) or self.opts[
            'type'] not in Parameter.supportedTypes:
            raise Exception("Parameter \"%s\" must have a valid string type." % name)

        # for opt in opts:
        #     if opt not in Parameter.suppertedAttributes:
        #         raise Exception("Parameter \"%s\" has unknown attribute type: %s." % (name, opt))

        if self.opts['type'] != 'group':
            if (('set' in self.opts) or ('get' in self.opts)) and ('value' in self.opts):
                raise Exception(
                    "Use set/get or value, not both simultaneously in parameter \"%s\". If an action is needed, use the action option." % name)

            if not (
                                'set' in self.opts or 'get' in self.opts or 'value' in self.opts or 'action' in self.opts or 'linked' in self.opts):
                raise Exception(
                    "Useless parameter \"%s\" because no set/get/value/action/linked option is defined." % name)

            if 'set' in self.opts and (not ('get' in self.opts)):
                raise Exception("Option set and get should be used together in parameter \"%s\"." % name)

            if 'get' in self.opts and (not 'set' in self.opts) and ('readonly' in self.opts) and self.opts[
                'readonly'] == False:
                raise Exception("Parameters \"%s\" has get and no set. Should be marked as readonly." % name)

            if self.opts.get("type", None) == "list":
                self.opts['limits'] = opts['values']

            if 'set' in self.opts:
                self.sigValueChanged.connect(self.opts['set'])
                self.opts['set'] = None

            if 'get' in self.opts:
                self.opts['get'] = util.WeakMethod(self.opts['get'])

            if 'action' in self.opts:
                self.opts['action'] = util.WeakMethod(self.opts['action'])

            if "default" not in self.opts:
                self.opts["default"] = self.getValue()
                self.setValue(self.opts["default"], init=True)
        else:
            if 'addLoadSave' in self.opts and self.opts["addLoadSave"]:
                self.opts["addLoadSave"] = (self.load, self.save)

        self.childs = []
        self.ignoredChildren = self.opts.pop("children", [])
        if Parameter.usePyQtGraph and self.opts["type"] != "menu":
            self.setupPyQtGraphParameter()
        self.keys = {}
        if ignoreChildren is False:
            self.addChildren(self.ignoredChildren)
            self.ignoredChildren = []

    def getName(self):
        return self.opts["name"]

    def getType(self):
        return self.opts["type"]

    def getTip(self):
        return self.opts["tip"]

    def getAction(self):
        return self.opts["action"]

    def getOpts(self):
        return self.opts

    def getValue(self, default=None):
        """Return the internal or external value of the parameter"""
        try:
            val = self.opts.get("get", None)
            if val is None:
                return self.opts.get("value", default)
            else:
                return val()
        except Exception as e:
            raise type(e)(type(e)(e.message +
                                   ' happens on parameter "%s"' % self.getName())).with_traceback(sys.exc_info()[2])

    def getKeyFromValue(self, value):
        """Return the key used to set list type parameters"""
        if self.opts["type"] == "list":
            limits = self.opts["limits"]
            if isinstance(limits, dict):
                try:
                    return list(limits.keys())[list(limits.values()).index(value)]
                except ValueError:
                    ValueError(
                        "Error: Value " + str(value) + " is not valid in Parameter \"" + self.getName() + "\". Options are: " + str(
                            limits))
        return value

    def getValueKey(self):
        """Return the key used to set list type parameters"""
        return self.getKeyFromValue(self.getValue())

    def addChildren(self, children):
        """Add a list of children to the current paramenter"""
        addedChildren = []
        for child in children:
            addedChildren.append(Parameter(self, ignoreChildren=True, **child))
            self.append(addedChildren[-1])
        for child in addedChildren:  # Prevent children being added out of order
            child.addChildren(child.ignoredChildren)
            child.ignoredChildren = []

    def append(self, child):
        """Add one child"""
        if child is None:
            return

        if child.getName() in self.keys:
            self.keys[child.getName()].remove()

        self.keys[child.getName()] = child
        self.childs.append(child)

        if 'key' in child.getOpts():
            self.keys[child.getOpts()["key"]] = child
        child.setParent(self)
        if child.opts["type"] != "menu":
            self.sigChildAdded.emit(child)
        self.sigParametersChanged.emit()

    def setValue(self, value, blockSignal=None, blockAction=False, init=False, ignoreReadonly=False, echo=True,
                 addToList=False):
        """
        Set the parameter value. External values are updated using signals.

        Arguments:
        ignoreReadonly - force readonly parameter to be updated.
        blockSignal    - used to prevent already changed valued of being set again, causing infinite loops.
        blockAction    - prevents action callback of being called.
        init           - used internally to initialize the parameter.
        echo           - enables/disables broadcasting the changes.
        addToList      - add given value to list of valid values if not already present
        """
        if not ignoreReadonly and not init and self.readonly():
            raise ValueError("Parameter \"%s\" is currently set to read only." % self.getName())
        limits = self.opts.get("limits", None)
        type = self.opts["type"]
        if type == "group":
            return

        if addToList:
            newlimits = copy.copy(limits)
            newlimits[value.getName()] = value
            self.setLimits(newlimits)

        elif limits is not None and not self.invalid:
            if (type == "list" and
                    ((isinstance(limits, dict) and value not in list(limits.values())) or \
                             (not isinstance(limits, dict) and value not in limits))
                ) or \
                    (type == "bool" and value not in [True, False]) or \
                    ((type == "int" or type == "float") and (value < limits[0] or value > limits[1])) or \
                    (type == "rangegraph" and (value[1] - value[0] != -1) and (
                                    value[0] < limits[0] or value[0] > limits[1] or value[1] < limits[0] or value[1] >
                        limits[1])):
                if isinstance(limits, dict) and value in list(limits.keys()):
                    value = limits[value]
                else:
                    #raise ValueError("Value %s out of limits (%s) in parameter \"%s\"" % (str(value), str(limits), self.getName()))
                    logging.info("Value %s out of limits (%s) in parameter \"%s\"" % (str(value), str(limits), self.getName()))

        try:
            if blockSignal is not None:
                self.sigValueChanged.disconnect(blockSignal)

            if "value" in self.opts:
                self.opts["value"] = value
            if 'psync' in self.opts and self.opts['psync'] == False:
                self.sigValueChanged.emit(value)
            else:
                self.sigValueChanged.emit(value, blockSignal=self.setValue)
        finally:
            if blockSignal is not None:
                self.sigValueChanged.connect(blockSignal)

        if self.previousValue is not None and self.previousValue() is not None:
            self.previousValue().getParams().hide()

        if isinstance(value, Parameterized):
            value.getParams().show()
            self.previousValue = weakref.ref(value)

        if not init:
            if not blockAction:
                self.callAction()
            if isinstance(limits, dict):
                for k, v in limits.items():
                    if v == value:
                        value = k

            if echo and not self.opts.get("echooff", False) and not self.readonly() and Parameter.scriptingOutput and Parameter.usePyQtGraph:
                path = self.getPath()
                if path is not None:
                    print(str(path + [value]) + ",", file=Parameter.scriptingOutput)

    def callLinked(self):
        for name in self.opts.get("linked", []):
            linked = self.parent.getChild(name)
            if linked is not None:
                linked.sigValueChanged.emit(linked.getValue(), blockSignal=None)

    def callAction(self):
        act = self.opts.get("action", None)
        if act is not None:
            act(self)
            if Parameter.scriptingOutput and Parameter.usePyQtGraph:
                path = self.getPath()
                if path is not None:
                    print((str(path + [None]) + ","), file=Parameter.scriptingOutput)
        self.callLinked()

    def setDefault(self, default):
        """Set the default value"""
        self.opts['default'] = default
        self.sigOptionsChanged.emit(default=default)

    def setLimits(self, limits):
        """Change the limits. Invalid interval limits are hidden."""
        self.opts['limits'] = limits
        type = self.opts["type"]
        if (type == "int" or type == "float" or type == "rangegraph" or type == "range") and limits[0] > limits[1]:
            self.invalid = True
            self.sigOptionsChanged.emit(visible=False)
        else:
            self.invalid = False
            self.sigOptionsChanged.emit(visible=self.isVisible())
            self.sigLimitsChanged.emit(list(limits.keys()) if isinstance(limits, dict) else limits)

    def readonly(self):
        return self.opts.get('readonly', False)

    def setReadonly(self, readonly=True):
        self.opts["readonly"] = readonly
        self.sigOptionsChanged.emit(readonly=readonly)

    def hide(self):
        self.show(False)

    def show(self, show=True):
        self.opts['visible'] = show
        if not self.invalid:
            self.sigOptionsChanged.emit(visible=show)
        self.sigParametersChanged.emit()

    def isVisible(self):
        return self.opts['visible']

    def remove(self):
        """Detaches the parameter from its parent"""
        if self.parent is None:
            return
        self.parent.removeChild(self)

    def delete(self):
        """Deletes itself (makes the GC job easier removing cicles). WARNING: Can't be called again!!!"""
        self.remove()
        for c in self.childs:
            c.delete()
        self.sigValueChanged.disconnectAll()
        self.sigLimitsChanged.disconnectAll()
        self.sigOptionsChanged.disconnectAll()
        self.sigChildAdded.disconnectAll()
        self.sigChildRemoved.disconnectAll()
        if Parameter.usePyQtGraph:
            # if hasattr(self._PyQtGraphParameter, "sigActivated"):
            #     self._PyQtGraphParameter.sigActivated.disconnect()
            # self._PyQtGraphParameter.sigValueChanged.disconnect()
            self._PyQtGraphParameter = None
        self.deregister()
        self.previousValue = None
        self.opts.clear()

    def clearChildren(self):
        """Remove all children."""
        for ch in self.childs:
            self.removeChild(ch)

    def removeChild(self, child):
        """Remove the specified child."""
        if child is None:
            return

        self.keys.pop(child.getOpts()["name"])
        try:
            self.keys.pop(child.getOpts()["key"])
        except KeyError:
            pass

        if child.opts["type"] != "menu":
            self.sigChildRemoved.emit(child)
        self.childs.remove(child)
        child.parent = None
        self.sigParametersChanged.emit()

    def getChild(self, nameOrPath):
        """Return the paramenter child with the given name/path."""
        if isinstance(nameOrPath, list) or isinstance(nameOrPath, tuple):
            item = self.keys.get(nameOrPath[0], None)

            if len(nameOrPath) == 1 or item is None:
                return item
            else:
                return item.getChild(nameOrPath[1:])
        else:
            try:
                return self.keys[nameOrPath]
            except:
                raise KeyError("Could not find parameter with key %s. Options are: %s" % (nameOrPath, str(self.keys)))

    def getPyQtGraphParameter(self):
        """Return the PyQtGraph Parameter if it exists."""
        if hasattr(self, "_PyQtGraphParameter"):
            return self._PyQtGraphParameter
        return None

    def setupPyQtGraphParameter(self):
        """Creates a PyQtGraph Parameter and keeps it synchronized"""
        opts = {}
        opts.update(self.getOpts())
        if "default" in self.opts:
            opts["value"] = opts["default"]
        if "limits" in self.opts and isinstance(self.opts["limits"], dict):
            opts['limits'] = list(opts['limits'].keys())
            opts["value"] = self.getKeyFromValue(opts["default"])
            del opts['values']
        self._PyQtGraphParameter = pyqtgraphParameter.create(**opts)
        if hasattr(self._PyQtGraphParameter, "sigActivated"):
            self._PyQtGraphParameter.sigActivated.connect(self.callAction)
        self.sigChildRemoved.connect(lambda c: self._PyQtGraphParameter.removeChild(c.getPyQtGraphParameter()))
        self.sigChildAdded.connect(lambda c: self._PyQtGraphParameter.addChild(c.getPyQtGraphParameter()))
        sigValueUpdatedAdapter = lambda _, v: self.setValue(v, sigSetValueAdapter)
        self._PyQtGraphParameter.sigValueChanged.connect(sigValueUpdatedAdapter)
        sigSetValueAdapter = lambda v, blockSignal=None: self._PyQtGraphParameter.setValue(self.getKeyFromValue(v),
                                                                                      sigValueUpdatedAdapter)
        self.sigValueChanged.connect(sigSetValueAdapter)
        self.sigLimitsChanged.connect(self._PyQtGraphParameter.setLimits)
        self.sigOptionsChanged.connect(self._PyQtGraphParameter.setOpts)

    def setParent(self, parent):
        self.parent = parent

    def getRoot(self):
        """Return the root Parameter."""
        return self if self.parent is None else self.parent.getRoot()

    def getPath(self):
        """Return the path to the root."""
        if self in list(Parameter.registeredParameters.values()):
            path = []
        elif self.parent is None:
            return None
        else:
            path = self.parent.getPath()
        if path is not None:
            path.append(self.opts["name"])
        return path

    def stealDynamicParameters(self, parent):
        """In list type parameters, append each of the Parameterized objects to the parent argument."""
        if self.opts.get("type", None) == "list" and isinstance(self.opts["values"], dict):
            for value in self.opts["values"].values():
                if isinstance(value, Parameterized):
                    parent.append(value.getParams())
                    value.getParams().show(self.getValue() == value)

    def refreshAllParameters(self):
        """
        Refreshes (calls set(get()) ) all the parameters in the hierarchy.
        In list type parameters, append each of the Parameterized objects to the parent, root or child hierarchy.
        """
        if self.opts.get("type", None) == "list" and isinstance(self.opts["values"], dict):
            for value in self.opts["values"].values():
                if isinstance(value, Parameterized):
                    mode = self.opts.get("childmode", None)
                    if mode == "parent":
                        self.parent.append(value.getParams())
                        value.getParams().hide()
                    elif mode == "root":  # Warning, this path is relative... can't call init more than once and sub-modules can't repeat name
                        self.getRoot().append(value.getParams())
                        value.getParams().hide()
                    elif mode == "child":
                        self.append(value.getParams())
                        value.getParams().hide()

        self.setValue(self.getValue(), init=True)
        for child in self.childs:
            child.refreshAllParameters()

    def init(self):
        self.refreshAllParameters()

    def _getAllParameters(self, type=None):
        ret = []
        if self.isVisible():
            if self.opts.get("type", None) == type or type == None:
                ret.append(self)
            for child in self.childs:
                ret.extend(child._getAllParameters(type))
        return ret

    @classmethod
    def getAllParameters(cls, type=None):
        """Return a list with all parameters with a given type in the hierarchy."""
        ret = []
        for p in cls.registeredParameters.values():
            parameters = p._getAllParameters(type)
            [ret.append(param) for param in parameters if param not in ret]
        return ret

    def register(self):
        """Makes it accessible from the root when calling setParameter()"""
        Parameter.registeredParameters[self.getName()] = self
        return self

    def deregister(self):
        """Deregister a registered parameter. Ignores if it is already deregistered."""
        Parameter.registeredParameters.pop(self.getName(), None)

    def toString(self, level, onlyVisibles=False):
        lastLevel = level
        ret = ""
        if self.getType() != "group" and self.getType() != "action" and self.getType() != "menu" and self.getType() != "label":
            if (onlyVisibles and self.isVisible()) or not (onlyVisibles or self.readonly()):
                ret += self.getName() + " = " + str(self.getValueKey()) + "\n"

        if not onlyVisibles or self.isVisible():
            for child in self.childs:
                if lastLevel != level+1:
                    ret += ("[" * (level + 1)) + self.getName() + ("]" * (level + 1)) + "\n"
                    lastLevel = level+1
                try:
                    txt, lastLevel = child.toString(lastLevel, onlyVisibles)
                    ret += txt
                except:
                    logging.info('Info: Could not read parameter %s. Ignoring it...' % str(child.getPath()))
                    pass
        return ret, lastLevel

    def __str__(self):
        txt, _ = self.toString(0)
        return txt

    def save(self, fname, onlyVisibles=False):
        f = open(fname, 'w')
        txt, _ = self.toString(0, onlyVisibles=False)
        f.write(txt)

    @classmethod
    def saveRegistered(cls, fname, onlyVisibles=False):
        f = open(fname, 'w')
        for p in cls.registeredParameters.values():
            if 'addLoadSave' in p.opts and p.opts['addLoadSave'] is not False:
                txt, _ = p.toString(0, onlyVisibles)
                f.write(txt)

    def load(self, fname):
        f = open(fname, 'r')
        path = []
        foundCorrectSection = False
        for lineNum, line in enumerate(f):
            level = 0
            for p in range(0, len(line)):
                if line[p] == "[":
                    level += 1
                else:
                    break
            if level != 0:
                path = path[0:level - 1]
                if level >= len(path):
                    if level > len(path) + 1:
                        raise Warning(
                            'Error reading file %s, line %d. Group hierarchy missing.' % (fname, lineNum))
                    path.append(line[level:-(level + 1)])
            else:
                if path[0] != self.getName():
                    continue
                foundCorrectSection = True
                separator = line.find("=")
                value = line[separator + 1:-1].strip()
                param = path[1:] + [line[0:separator].strip()]
                child = self.getChild(param)
                if child is None:
                    logging.warning('Parameter "%s" in line %d not found. Ignoring it.' % (str(param), lineNum))
                    continue

                if child.getType() == "int":
                    value = int(value)
                elif child.getType() == "float":
                    value = float(value)
                elif child.getType() == "menu" or child.getType() == "label" or child.getType() == "color":
                    continue
                elif child.getType() == "str" or child.getType() == "text" or child.getType() == "action" or \
                                child.getType() == "file" or child.getType() == "filelist":
                    pass
                elif child.getType() == "list" and isinstance(child.opts["limits"], dict):
                    value = child.opts["limits"][value]
                elif value != "":
                    try:
                        value = eval(value)
                    except:
                        pass
                if not child.readonly():
                    child.setValue(value)
                else:
                    if str(child.getValue()) != str(value):
                        logging.info('Parameter %s in line %d is readonly and value being set is different than the current one.' % (child.getName(), lineNum))
        if not foundCorrectSection:
            raise Warning('Could not found section "%s" in file: %s' % (self.getName(), fname))

    @classmethod
    def findParameter(cls, path):
        """
        Find a registered parameter based on a list (used for scripting).
        """
        child = cls.registeredParameters.get(path[0], None)
        if child is None:
            raise KeyError("Parameter not found: %s" % str(path))
        if len(path) > 1:
            return child.getChild(path[1:])
        else:
            return child

    @classmethod
    def getParameter(cls, path):
        """Return the value of a registered parameter"""
        return cls.findParameter(path).getValueKey()

    @classmethod
    def setParameter(cls, parameter, echo=False, blockSignal=False):
        """
        Sets a parameter based on a list (used for scripting).
        The first elements are the path and the last is the value.
        """
        path = parameter[:-1]
        value = parameter[-1]

        child = cls.findParameter(path)
        if isinstance(child.getOpts().get("values", None), dict):
            try:
                value = child.getOpts()["values"][value]
            except KeyError:
                raise ValueError("Invalid value '%s' for parameter '%s'.\nValid values: %s" %
                                 (value, str(parameter), list(child.getOpts()["values"].keys())))
        child.setValue(value, echo=echo)


def setupSetParam(parameter):
    """
    Decorator that should be used in the set methods specified in the parameter declaration.
    It synchronizes the parameter value when the set method is called directly.
    The blockSignal argument can be used to avoid this behavior when, for instance, you can't do that because the
    parameter wasn't created yet
    """

    def func_decorator(func):
        @functools.wraps(func)
        def func_wrapper(*args, **kargs):
            blockSignal = kargs.get("blockSignal", None)
            if "blockSignal" in kargs:
                del kargs["blockSignal"]
            if blockSignal is None:
                if parameter != "":
                    tmp = args[0].findParam(parameter)
                    tmp.setValue(args[1], blockSignal=tmp.opts["set"], **kargs)
            # todo - use inspect to remove things from kargs that are handled by setvalue above
            if "addToList" in kargs:
                del kargs["addToList"]
            func(*args, **kargs)

        return func_wrapper
        func_wrapper.__wrapped__ = func

    return func_decorator


if __name__ == '__main__':
    from pyqtgraph.Qt import QtGui
    from pyqtgraph.parametertree import ParameterTree

    app = QtGui.QApplication([])


    class submodule(Parameterized):
        def __init__(self, d):
            super(submodule, self).__init__()
            moreparams = [
                {'name': 'Value', 'type': 'list', 'values': [2, 3, 4], 'value': 3, 'action': self.action,
                 'linked': ['Linked Value (1/Value)']},
                {'name': 'Linked Value (1/Value)', 'type': 'float', 'default': 1 / 3, 'get': self.getLinked}
            ]
            self.params = Parameter(name='Sub-module %d' % d, type='group', children=moreparams)

        def action(self, e):
            print("action %s" % e.getValue())

        def getLinked(self):
            return 1.0 / float(self.params.getChild('Value').getValue())


    class module(Parameterized):
        def __init__(self, d):
            super(module, self).__init__()
            values = {'sm1': submodule(1), 'sm2': submodule(2), 'sm3': submodule(3)}
            self.sm = values['sm2']
            moreparams = [
                {'name': 'SubModule', 'type': 'list',
                 'values': values, 'set': self.setSubmodule, 'get': self.getSubmodule,
                 'childmode': "child",
                 },
            ]
            self.params = Parameter(name='Module %d' % d, type='group', children=moreparams)

        def getSubmodule(self):
            return self.sm

        @setupSetParam("SubModule")
        def setSubmodule(self, sm):
            self.sm = sm


    class maintest(Parameterized):
        def __init__(self):
            super(maintest, self).__init__()
            self.values = {'module 1': module(1), 'module 2': module(2), 'module 3': module(3)}
            self.module = self.values['module 2']
            p = [
                {'name': 'Module', 'type': 'list',
                 'values': self.values,
                 'set': self.setmodule,
                 'get': self.getmodule,
                 'childmode': "root",
                 'help': '%namehdr%Boatload of text is possible here. Can use markup too with external help window.'},
                {'name': 'Rocks to Skips', 'type': 'int', 'value': 0, 'help': 'Another help example', 'helpwnd': None,
                 'linked': ['1/Rocks to Skips']},
                {'name': '1/Rocks to Skips', 'type': 'float', 'help': 'Another help example', 'helpwnd': None,
                 'get': self.getLinked}
            ]
            self.params = Parameter(name='Root', type='group', children=p).register()
            self.params.init()

            self.t = ParameterTree()
            self.t.addParameters(self.params.getPyQtGraphParameter())

            self.test = Parameter(name='Test', type='group')
            self.test.addChildren([
                {'name': 'baud', 'type': 'int', 'key': 'baud', 'limits': (500, 2000000), 'value': 38400}
            ])

            self.params.append(self.test)
            self.test.remove()
            self.test.getChild("baud").setValue(500)
            self.params.append(self.test)

            self.t2 = ParameterTree()
            self.params2 = Parameter(name='Root', type='group')
            self.params.getChild("Module").stealDynamicParameters(self.params2)
            self.t2.addParameters(self.params2._PyQtGraphParameter)
            self.params.save("abcde.txt")
            self.params.load("abcde.txt")

        def printhelp(self, msg, obj):
            print(msg)

        def getmodule(self):
            return self.module

        @setupSetParam("Module")
        def setmodule(self, module):
            print("Changing Module")
            self.module = module

        def getLinked(self):
            try:
                return 1.0 / float(self.params.getChild('Rocks to Skips').getValue())
            except:
                return 0


    m = maintest()

    t = m.t
    t.show()
    t.setWindowTitle('pyqtgraph example: Parameter Tree')
    t.resize(400, 800)
    t2 = m.t2
    t2.show()
    t2.setWindowTitle('pyqtgraph example: Parameter Tree')
    t2.resize(400, 800)
    # Parameter.setParameter(['Root', 'Module', 'module 3'])
    # m.setmodule(m.values['module 1'])

    QtGui.QApplication.instance().exec_()
    # pg.systemInfo()
    # t2 = ParameterTree()
    # t2.show()
    # root = pyqtgraphParameter.create(name='Root', type='group')
    # t2.addParameters(root)
    # par = pyqtgraphParameter.create(name='Par1', type='group')
    # par.addChildren([
    #     {'name':'test', 'type':'int', 'limits':(500, 2000000), 'value':38400}
    # ])
    # root.addChild(par)
    # root.removeChild(par)
    # root.addChild(par)
    #
    # QtGui.QApplication.instance().exec_()



#!/usr/bin/env python
"""
AES Block Cipher.

Performs single block cipher decipher operations on a 16 element list of integers.
These integers represent 8 bit bytes in a 128 bit block.
The result of cipher or decipher operations is the transformed 16 element list of integers.

Running this file as __main__ will result in a self-test of the algorithm.

Algorithm per NIST FIPS-197 http://csrc.nist.gov/publications/fips/fips197/fips-197.pdf

Thanks to serprex for many optimizations in this code. For even more, see his github fork of this project.

Copyright (c) 2010, Adam Newman http://www.caller9.com/
Demur Rumed https://github.com/serprex
Licensed under the MIT license http://www.opensource.org/licenses/mit-license.php
"""
__author__ = "Adam Newman"

from . import aes_tables

class AESCipher:
    """Perform single block AES cipher/decipher"""

    def __init__ (self, expanded_key):
        #Store epanded key
        self._expanded_key = expanded_key

        #Number of rounds determined by expanded key length
        self._Nr = int(len(expanded_key) / 16) - 1

    def _sub_bytes (self, state):
        #Run state through sbox
        for i,s in enumerate(state):state[i]=aes_tables.sbox[s]

    def _i_sub_bytes (self, state):
        #Run state through inverted sbox
        for i,s in enumerate(state):state[i]=aes_tables.i_sbox[s]

    def _shift_row (self, row, shift):
        #Circular shift row left by shift amount
        row+=row[:shift]
        del row[:shift]
        return row

    def _i_shift_row (self, row, shift):
        #Circular shift row left by shift amount
        row+=row[:shift]
        del row[:4+shift]
        return row

    def _shift_rows (self, state):
        #Extract rows as every 4th item starting at [1..3]
        #Replace row with shift_row operation
        for i in 1,2,3:
            state[i::4] = self._shift_row(state[i::4],i)

    def _i_shift_rows (self, state):
        #Extract rows as every 4th item starting at [1..3]
        #Replace row with inverse shift_row operation
        for i in 1,2,3:
            state[i::4] = self._i_shift_row(state[i::4],-i)

    def _mix_column (self, column, inverse):
        #Use galois lookup tables instead of performing complicated operations
        #If inverse, use matrix with inverse values
        g0,g1,g2,g3=aes_tables.galI if inverse else aes_tables.galNI
        c0,c1,c2,c3=column
        return (
            g0[c0]^g1[c1]^g2[c2]^g3[c3],
            g3[c0]^g0[c1]^g1[c2]^g2[c3],
            g2[c0]^g3[c1]^g0[c2]^g1[c3],
            g1[c0]^g2[c1]^g3[c2]^g0[c3])

    def _mix_columns (self, state, inverse):
        #Perform mix_column for each column in the state
        for i,j in (0,4),(4,8),(8,12),(12,16):
            state[i:j] = self._mix_column(state[i:j], inverse)

    def _add_round_key (self, state, round):
        #XOR the state with the current round key
        for k,(i,j) in enumerate(zip(state, self._expanded_key[round*16:(round+1)*16])):state[k]=i^j

    def cipher_block (self, state):
        """Perform AES block cipher on input"""
        #PKCS7 Padding
        state=state+[16-len(state)]*(16-len(state))#Fails test if it changes the input with +=

        self._add_round_key(state, 0)

        for i in range(1, self._Nr):
            self._sub_bytes(state)
            self._shift_rows(state)
            self._mix_columns(state, False)
            self._add_round_key(state, i)

        self._sub_bytes(state)
        self._shift_rows(state)
        self._add_round_key(state, self._Nr)
        return state

    def decipher_block (self, state):
        """Perform AES block decipher on input"""
        #null padding. Padding actually should not be needed here with valid input.
        state=state+[0]*(16-len(state))

        self._add_round_key(state, self._Nr)

        for i in range(self._Nr - 1, 0, -1):
            self._i_shift_rows(state)
            self._i_sub_bytes(state)
            self._add_round_key(state, i)
            self._mix_columns(state, True)

        self._i_shift_rows(state)
        self._i_sub_bytes(state)
        self._add_round_key(state, 0)
        return state



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

# USED BY SMARTCARD, if we want to keep that

from ._base import SimpleSerialTemplate
import time
from chipwhisperer.common.utils.parameter import setupSetParam


class SimpleSerial_ChipWhisperer(SimpleSerialTemplate):
    _name = "ChipWhisperer"
    CODE_READ       = 0x80
    CODE_WRITE      = 0xC0
    ADDR_DATA       = 33
    ADDR_LEN        = 34
    ADDR_BAUD       = 35

    def __init__(self):
        SimpleSerialTemplate.__init__(self)
        self._regVer = 0
        self.params.addChildren([
            {'name':'TX Baud', 'key':'txbaud', 'type':'int', 'limits':(0, 1E6), 'default':38400, 'get':self.txBaud, 'set':self.setTxBaud},
            {'name':'RX Baud', 'key':'rxbaud', 'type':'int', 'limits':(0, 1E6), 'default':38400, 'get':self.rxBaud, 'set':self.setRxBaud},
            {'name':'Stop-Bits', 'key':'stopbits', 'type':'list', 'values':{'1':1, '2':2}, 'default':1, 'get':self.stopBits,
                            'set':self.setStopBits, 'readonly':True},
            {'name':'Parity', 'key':'parity', 'type':'list', 'values':{'None':'n', 'Even':'e'}, 'default':'n', 'get':self.parity,
                            'set':self.setParity, 'readonly':True},
        ])

    def systemClk(self):
        return 30E6

    @setupSetParam("TX Baud")
    def setTxBaud(self, baud):
        breg = (baud * 4096 + self.systemClk() / 32) / (self.systemClk() / 16)
        breg = int(round(breg))
        self.setTxBaudReg(breg)

    @setupSetParam("RX Baud")
    def setRxBaud(self, baud):
        breg = (baud * 8 * 512 + self.systemClk() / 255) / (self.systemClk() / 128)
        breg = int(round(breg))
        self.setRxBaudReg(breg)

    def txBaud(self):
        breg = self.txBaudReg()
        baud = ((breg * (self.systemClk() / 16)) - (self.systemClk() / 32)) / 4096
        return baud

    def rxBaud(self):
        breg = self.rxBaudReg()
        baud = ((breg * (self.systemClk() / 128)) - (self.systemClk() / 255)) / 512
        baud = baud / 8
        return baud

    def checkVersion(self):
        """Check for newer version of register set - this MUST be called before any
           calls to setxxBaud, as otherwise these bits get blasted away
        """
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)

        if data is None:
            raise IOError("ChipWhisperer-SER Module not found, check you are using updated FPGA Bitstream")

        if(data[3] & 0b11000000) == 0b11000000:
            self._regVer = 1
            self.findParam('stopbits').setReadonly(False)
            self.findParam('parity').setReadonly(False)

    def setTxBaudReg(self, breg):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        data[2] = breg & 0xff
        data[3] = (breg >> 8) & 0xff
        self.oa.sendMessage(self.CODE_WRITE, self.ADDR_BAUD, data)

    def setRxBaudReg(self, breg):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        data[0] = breg & 0xff
        highbyte = (breg >> 8) & 0x3F
        data[1] = (data[1] & 0xC0) | highbyte
        self.oa.sendMessage(self.CODE_WRITE, self.ADDR_BAUD, data)

    def txBaudReg(self):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        breg = data[2] | ((data[3] & 0x3F) << 8)
        return breg

    def rxBaudReg(self):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        breg = data[0] | ((data[1] & 0x3F) << 8)
        return breg

    def stopBits(self):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        if data[1] & (1 << 6):
            return 2
        else:
            return 1
        self.oa.sendMessage(self.CODE_WRITE, self.ADDR_BAUD, data)

    @setupSetParam("Stop-Bits")
    def setStopBits(self, stopbits):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        if stopbits == 1:
            data[1] = data[1] & ~(1 << 6)
        else:
            data[1] = data[1] | (1 << 6)
        self.oa.sendMessage(self.CODE_WRITE, self.ADDR_BAUD, data)

    def parity(self):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        if data[1] & (1 << 7):
            return 'e'
        else:
            return 'n'

    @setupSetParam("Parity")
    def setParity(self, par):
        data = self.oa.sendMessage(self.CODE_READ, self.ADDR_BAUD, maxResp=4)
        if par == 'e':
            data[1] = data[1] | (1 << 7)
        else:
            data[1] = data[1] & ~(1 << 7)
        self.oa.sendMessage(self.CODE_WRITE, self.ADDR_BAUD, data)

    def debugInfo(self, lastTx=None, lastRx=None, logInfo=None):
        #if self.debugLog is not None:
        #    self.debugLog(lastTx, lastRx)
        pass

    def hardware_write(self, string):
        for s in string:
            if isinstance(string, str):
                d = bytearray(s, 'latin-1')
            else:
                d = [s]
            #print "%x"%d[0]
            self.oa.sendMessage(self.CODE_WRITE, self.ADDR_DATA, d, Validate=False)

            self.debugInfo(s)

    def hardware_inWaiting(self):
        resp = self.oa.sendMessage(self.CODE_READ, self.ADDR_LEN, Validate=False, maxResp=2)
        resp = resp[1]
        # print "%d waiting"%resp
        return resp

    def hardware_read(self, num=0, timeout=100):
        waiting = self.inWaiting()
        data = bytearray()

        #TODO: why is this needed? Some garbage at front...
        # num = num + 1

        while (len(data) < num) and (timeout > 1):
            if waiting > 0:
                resp = self.oa.sendMessage(self.CODE_READ, self.ADDR_DATA, Validate=False, maxResp=1)
                if resp:
                    data.append(resp[0])
            else:
                time.sleep(0.001)
                timeout = timeout - 1
            waiting = self.inWaiting()

        if timeout <= 0:
            print("CW Serial timed out")

        #TODO: fix removing garbage at front
        # result = data[1:(len(data)+1)]
        result = data
        result = result.decode('latin-1')
        self.debugInfo(lastRx=result)
        return result

    def flush(self):
        waiting = self.inWaiting()
        while waiting > 0:
            self.oa.sendMessage(self.CODE_READ, self.ADDR_DATA, Validate=False, maxResp=1)
            waiting = self.inWaiting()

    def flushInput(self):
        self.flush()

    def con(self, scope=None):
        if not scope or not hasattr(scope, "qtadc"): Warning("You need a scope with OpenADC connected to use this Target")

        self.oa = scope.qtadc.sc
        scope.connectStatus.connect(self.dis)
        # Check first!
        self.checkVersion()
        self.params.refreshAllParameters()
        self.connectStatus.setValue(True)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.common.utils.util import camel_case_deprecated

class AcqKeyTextPattern_Base(object):
    _name = "Key/Text Pattern"

    def __init__(self):
        self._key_len = 16
        self._text_len = 16
        self._key = None
        self._textin = None

    def setTarget(self, target):
        pass

    @property
    def key_len(self):
        return self._key_len

    @key_len.setter
    def key_len(self, n):
        self._key_len = n

    @property
    def text_len(self):
        return self._text_len

    @text_len.setter
    def text_len(self, n):
        self._text_len = n

    @property
    def fixed_key(self):
        """Generate fixed key (True) or not (False).

        :Getter: Return True if using fixed key or False if not.

        :Setter: Set whether to use fixed key (True) or not (False).
        """
        raise NotImplementedError("Target does not allow key to be changed to fixed/unfixed")

    @fixed_key.setter
    def fixed_key(self, enabled):
        raise NotImplementedError("Target does not allow key to be changed to fixed/unfixed")

    @property
    def fixed_text(self):
        """Generate fixed plaintext (True) or not (False).

        :Getter: Return True if using fixed plaintext or False if not.

        :Setter: Set whether to use fixed plaintext (True) or not (False).
        """
        raise NotImplementedError("Target does not allow plaintext to be changed to fixed/unfixed")

    @fixed_text.setter
    def fixed_text(self, enabled):
        raise NotImplementedError("Target does not allow plaintext to be changed to fixed/unfixed")

    def keyLen(self):
        return self._key_len

    def textLen(self):
        return self._text_len

    def validateKey(self):
        if len(self._key) != self.key_len:
            raise IOError("Key Length Wrong for Given Target, %d != %d" % (self.key_len, len(self.key)))

    def validateText(self):
        if len(self._textin) != self.text_len:
            raise IOError("Plaintext Length Wrong for Given Target, %d != %d" % (self.text_len, len(self.textin)))

    def _initPattern(self):
        """Perform any extra init stuff required. Called at the end of main init() & when target changed."""
        pass

    def setInitialKey(self, initialKey, binaryKey=False):
        pass

    def setInitialText(self, initialText, binaryText=False):
        pass

    def init(self, maxtraces):
        """Initialize ktp for trace runs

        Not required for all targets (i.e. a basic key text pair)
        """
        raise NotImplementedError("This target does not implement init")

    init_pair = init

    initPair = camel_case_deprecated(init_pair)

    def new_pair(self):
        """Called when a new encryption pair is requested"""
        raise NotImplementedError("This needs to be reimplemented")
    newPair = camel_case_deprecated(new_pair)

    def __str__(self):
        return "key: {}\ntext: {}".format(list(hex(b) for b in self._key), list(hex(b) for b in self._textin))

    def next(self):
        self.new_pair()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import SimpleSerialTemplate
from chipwhisperer.hardware.naeusb.serial import USART as CWL_USART

class SimpleSerial_ChipWhispererLite(SimpleSerialTemplate):
    _name = 'NewAE USB (CWLite/CW1200)'

    def __init__(self):
        SimpleSerialTemplate.__init__(self)
        self._baud = 38400
        self.cwlite_usart = None

    def close(self):
        pass

    def setBaud(self, baud):
        self._baud = baud
        if self.cwlite_usart:
            self.cwlite_usart.init(baud)
        else:
            logging.error('Baud rate not set, need to connect first')

    def baud(self):
        return self._baud

    def con(self, scope = None):
        if scope is None or not hasattr(scope, "qtadc"):
            Warning("You need a scope with OpenADC connected to use this Target")

        if hasattr(scope, 'qtadc'):
            ser = scope.qtadc.ser
        else:
            ser = scope._cwusb
        self.cwlite_usart = CWL_USART(ser)
        self.cwlite_usart.init(baud=self._baud)


    def hardware_inWaiting(self):
        bwait = self.cwlite_usart.inWaiting()
        if bwait == 127:
            logging.warning('SAM3U Serial buffers OVERRUN - data loss has occurred.')
        return bwait

    def hardware_write(self, string):
        self.cwlite_usart.write(string)

    def hardware_read(self, num, timeout=250):
        return self.cwlite_usart.read(num, timeout)


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
import logging

__author__ = "Colin O'Flynn"

import os.path
import sys
from PySide.QtGui import *
from PySide.QtCore import *
from chipwhisperer.common.utils import tracereader_dpacontestv3, tracereader_native
import numpy as np
from time import gmtime, strftime
from ._base import TraceContainer
from chipwhisperer.common.utils import qt_tweaks as QtFixes

class TraceContainerDPAv3(TraceContainer):
    _name = "DPAContestv3"

    def __init__(self):
        super(TraceContainerDPAv3, self).__init__()
        self.dir = "."

    def setDirectory(self, directory):
        self.dir = directory

        os.mkdir(directory)

    def prepareDisk(self):
        self.startTime = gmtime()
        self.setDirectory("capture-%s/"%strftime("%Y.%m.%d-%H.%M.%S", self.startTime))
        
        if os.path.exists(self.dir + "text_in.txt"):
            logging.info('Textin File exists!')
            return

        if os.path.exists(self.dir + "text_out.txt"):
            logging.info('Textout File exists!')
            return

        if os.path.exists(self.dir + "wave.txt"):
            logging.info('Wave File exists!')
            return

        if os.path.exists(self.dir + "key.txt"):
            logging.info('Key file exists!')
            return
        
        self.inf = open(self.dir + "text_in.txt", "w")
        self.outf = open(self.dir + "text_out.txt", "w")
        self.wavef = open(self.dir + "wave.txt", "w")
        self.keyf = open(self.dir + "key.txt", "w")
        
    def numPoints(self):
        return self._numPoints
        
    def addTrace(self, wave, textin, textout, key=None, dtype=np.double):
        self.wavelen = len(wave)
        self._numTraces = self._numTraces + 1
        self._numPoints = len(wave)

        for i in textin:
            self.inf.write('%2X '%i)
        self.inf.write('\n')

        for i in textout:
            self.outf.write('%2X '%i)
        self.outf.write('\n')

        for i in wave:
            iint = i * float(2**16)
            self.wavef.write('%d '%int(iint))
        self.wavef.write('\n')

        if key:
            for i in key:
                self.keyf.write('%2X '%i)
            self.keyf.write('\n')
        
    def copyTo(self, srcTraces=None):
        pass        
        
    def loadAllTraces(self, directory=None, prefix=""):
        pass

    def writeInfo(self):
        infofile = open(self.dir + "info.xml", "w")

        infofile.write('<?xml version="1.0" encoding="utf-8"?>\n')
        infofile.write('<WaveformInfo xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">\n')
        infofile.write('  <Date>%s</Date>'%strftime("%d/%m%Y %H:%M:%S", self.startTime))
        infofile.write('  <Operator>Smooth</Operator>\n')
        infofile.write('  <WaveType>PowerTrace</WaveType>\n')
        infofile.write('  <WaveFormat>System.Single[]</WaveFormat>\n')
        infofile.write('  <Instrument>Something</Instrument>\n')
        infofile.write('  <Module>AES</Module>\n')
        infofile.write('  <Cipher>AES</Cipher>\n')
        infofile.write('  <KeyLength>128</KeyLength>\n')
        infofile.write('  <TextWidth>128</TextWidth>\n')
        infofile.write('  <NumTrace>%d</NumTrace>\n'%self._numTraces)
        infofile.write('  <NumPoint>%d</NumPoint>\n'%self._numPoints)
        infofile.write('</WaveformInfo>\n')
        infofile.close()

    def writeSettings(self, settings):
        settingsfile = open(self.dir + "settings.txt", "w")
        settingsfile.write("OpenADC Capture Settings:\n")
        for s in settings:
            settingsfile.write("%s\n"%s)       
        settingsfile.close()

    def saveAllTraces(self):
        self.writeInfo()
        
    def closeAll(self, clearTrace=True, clearText=True, clearKeys=True):
        self.saveAllTraces()
        self.inf.close()
        self.outf.close()
        self.wavef.close()
        self.keyf.close()


class ImportDPAv3Dialog(QtFixes.QDialog):
    def __init__(self, parent=None):
        super(ImportDPAv3Dialog, self).__init__(parent)
        self.parent = parent
        
        self.setWindowTitle("DPA Contest v3 Trace Format Importer")
        layout = QVBoxLayout()
        self.setLayout(layout)

        ### Selection of info.xml to import

        fileL = QHBoxLayout()
        fileL.addWidget(QLabel("info.xml file:"))
        self.fileLE = QtFixes.QLineEdit()
        #self.fileLE.setEnabled(False)
        fileL.addWidget(self.fileLE)
        filePB = QPushButton("Select info.xml")
        filePB.clicked.connect(self.selectInfo)
        fileL.addWidget(filePB)

        setupGB = QGroupBox("File Selection")
        setupGB.setLayout(fileL)
        layout.addWidget(setupGB)

        ### Settings found in info.xml, can be changed by user

        self.LETraces = QtFixes.QLineEdit()
        self.LETraces.setEnabled(False)

        self.LEPoints = QtFixes.QLineEdit()
        self.LEPoints.setEnabled(False)

        self.LEScope = QtFixes.QLineEdit()
        self.LETargetHW = QtFixes.QLineEdit()
        self.LETargetSW = QtFixes.QLineEdit()
        self.LEDate = QtFixes.QLineEdit()

        self.LENotes = QtFixes.QLineEdit()

        settingsL = QGridLayout()
        settingsL.addWidget(QLabel("Traces:"), 0, 0)
        settingsL.addWidget(self.LETraces, 0, 1)

        settingsL.addWidget(QLabel("Points:"), 0, 2)
        settingsL.addWidget(self.LEPoints, 0, 3)

        settingsL.addWidget(QLabel("Date:"), 1, 0)
        settingsL.addWidget(self.LEDate, 1, 1)

        settingsL.addWidget(QLabel("TargetHW:"), 2, 0)
        settingsL.addWidget(self.LETargetHW, 2, 1)

        settingsL.addWidget(QLabel("TargetSW:"), 2, 2)
        settingsL.addWidget(self.LETargetSW, 2, 3)

        settingsL.addWidget(QLabel("ScopeName:"), 3, 0)
        settingsL.addWidget(self.LEScope, 3, 1)

        settingsL.addWidget(QLabel("Notes:"), 4, 0)
        settingsL.addWidget(self.LENotes, 4, 1)

        settingsGB = QGroupBox("Imported Parameters")
        settingsGB.setLayout(settingsL)
        layout.addWidget(settingsGB)


        ### Import Options
        importL = QGridLayout()

        importL.addWidget(QLabel("Target Dir:"), 0, 0)
        targetDirLE = QtFixes.QLineEdit()
        targetDirLE.setText("./traces/")
        targetDirLE.setEnabled(False)
        importL.addWidget(targetDirLE, 0, 1)

        self.prefixDirLE = QtFixes.QLineEdit()
        self.prefixDirLE.setText("001")
        importL.addWidget(QLabel("Prefix:"), 0, 2)
        importL.addWidget(self.prefixDirLE, 0, 3)
        
        importPB = QPushButton("Run Import")
        importPB.clicked.connect(self.doImport)
        importL.addWidget(importPB, 4, 0)

        importGB = QGroupBox("Import Options")
        importGB.setLayout(importL)
        layout.addWidget(importGB)

    def getTraceCfgFile(self):
        return self.parent.parent.cwp.traceslocation + "/" + "config_" + self.prefixDirLE.text() + ".cfg"
       
    def selectInfo(self):
        fname, _ = QFileDialog.getOpenFileName(self, 'Open file', QSettings().value("dpav3_last_file"), 'info.xml')
        if fname:
            QSettings().setValue("dpav3_last_file", fname)
            trimport = tracereader_dpacontestv3.tracereader_dpacontestv3()
            trimport.loadInfo(os.path.dirname(fname))
            self.trimport = trimport

            self.fileLE.setText(fname)

            self.LEPoints.setText("%d"%trimport.numPoint)
            self.LETraces.setText("%d"%trimport.numTrace)

            self.LEScope.setText(trimport.xmlroot.findall('Instrument')[0].text)
            self.LEDate.setText(trimport.xmlroot.findall('Date')[0].text)
            self.LETargetHW.setText(trimport.xmlroot.findall('Module')[0].text)
            self.LETargetSW.setText(trimport.xmlroot.findall('Cipher')[0].text)

            #May not have notes
            try:
                self.LENotes.setText(trimport.xmlroot.findall('Notes')[0].text)
            except:
                pass

            #You really need a project file open, but let user experiment without it...
            if self.parent.parent.cwp:
                self.validatePrefix()

    def validatePrefix(self, change=True):              
        tracedir = self.parent.parent.cwp.traceslocation
        configOK = False

        while configOK == False:
            if os.path.exists(self.getTraceCfgFile()):
                if change:
                    try:
                        cur = int(self.prefixDirLE.text())
                        cur = cur + 1
                        self.prefixDirLE.setText("%03d"%cur)
                    except:
                        change = False
                        
                if change == False:
                        msg = QMessageBox()
                        msg.setText("Settings Error")
                        msg.setInformativeText("Prefix already in use, try another prefix")
                        msg.setStandardButtons(QMessageBox.Ok)
                        msg.exec_()
                        return False
            else:
                configOK = True

        return True

    def doImport(self):
        """
        Import traces from DPAv3 Contest Format into internal native ChipWhisperer format.
        
        NOTE: This won't actually be required in the future... for now you have to convert
        into the CW format, but the idea is you can just read them from disk in the DPAv3 format.
        """
        
        if self.parent.parent.cwp == None:
            logging.warning('No project open for import? Hmm...')
            return

        if self.validatePrefix(False) == False:
            return
        
        ti = TraceContainer()
        ti.numTraces = int(self.LETraces.text())
        ti.numPoints = int(self.LEPoints.text())
        ti.date = self.LEDate.text()
        ti.scope = self.LEScope.text()
        ti.targetHW = self.LETargetHW.text()
        ti.targetSW = self.LETargetSW.text()
        ti.notes = self.LENotes.text()

        msg = QMessageBox()
        msg.setText("Message from ChipWhisperer")
        msg.setInformativeText("Importing, could take a while. This message dissappears when done")
        msg.setStandardButtons(0)
        msg.show()
        self.trimport.loadAllTraces()
    
        tmp = tracereader_native.tracereader_native()
        tmp.copyTo(self.trimport)
        msg.close()

        #Save file automatically
        tmp.saveAllTraces(self.parent.parent.cwp.traceslocation + "/", prefix=self.prefixDirLE.text() + "_")

        ti.prefix = self.prefixDirLE.text() + "_"
        
        ti.saveTrace(self.getTraceCfgFile())     
        self.close()



"""
Package containing all of the key text generation types

KTP:
* Basic - fixed/random key and text (default fixed key, random text)
* DPA - Applies mask to random plaintext, then randomly undoes it
* TVLATTest
"""
from .basic import AcqKeyTextPattern_Basic as Basic
from .dpahelper import AcqKeyTextPattern_DPA as DPA
from .tvlattest import AcqKeyTextPattern_TVLATTest as TVLATTest



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

import copy
import logging
import re
import numpy as np
from . import _cfgfile
from chipwhisperer.common.utils.parameter import Parameterized


class TraceContainer(Parameterized):
    """
    TraceContainer holds traces for the system to operate on. This can include both reading in traces for analysis, and
    writing traces to disk.
    
    This class is normally used as a base class for some specific format. For example the 'TraceFormatNative' class
    adds functions for reading/storing data in the 'native' ChipWhisperer format.
    """
    _name = "Trace Configuration"
    
    def __init__(self, configfile=None, project=None, default_setup=False):
        self.configfile = configfile
        self.fmt = None
        self.getParams().register()
        self.getParams().addChildren([
                {'name':'Config File', 'key':'cfgfile', 'type':'str', 'readonly':True, 'value':''},
                {'name':'Format', 'key':'format', 'type':'str', 'readonly':True, 'value':''},
        ])
        self.clear()
        self.project = project
        
        #If we have a project file, do specific setup related to project
        if project and default_setup:
            self.default_config_setup(project)

    def clear(self):
        self.config = _cfgfile.TraceContainerConfig(configfile=self.configfile)
        self.textins = []
        self.textouts = []
        self.keylist = []
        self.knownkey = None
        self.dirty = False
        self.tracedtype = np.double
        self.traces = None
        self.tracehint = 1
        self.pointhint = 0
        self._numTraces = 0
        self._isloaded = False

    def setDirty(self, dirty):
        self.dirty = dirty
        
    def updateConfigData(self):
        return

    def numPoints(self):
        try:        
            return self.traces.shape[1]
        except:
            return 0
        
    def setTraceBuffer(self, tracebuffer):
        """Reuse a trace buffer allocated elsewhere"""
        self.traces = tracebuffer

    def setTraceHint(self, traces):
        self.tracehint = traces
        
    def setPointHint(self, points):
        self.pointhint = points

    def numTraces(self):
        cfint = int(self.config.attr("numTraces"))
        if cfint != self._numTraces:
            self._numTraces = max(cfint, self._numTraces)
        return self._numTraces

    def add_trace(self, trace, textin, textout, key, dtype=np.double, channelNum=0):
        if channelNum!=0:
            raise NotImplementedError
        self.addWave(trace, dtype)
        self.addTextin(textin)
        self.addTextout(textout)
        self.addKey(key)
        self.project.trace_manager()._updateRanges()

    addTrace = add_trace

    def writeDataToConfig(self):
        self.config.setAttr("numTraces", self._numTraces)
        self.config.setAttr("numPoints", self.numPoints())      

    def addWave(self, trace, dtype=None):
        try:
            if self.traces is None:
                if dtype is None:
                    dtype = np.double
                self.tracedtype = dtype
                self.traces = np.zeros((self.tracehint, len(trace)), dtype=dtype)
                self.traces[self._numTraces][:] = trace
            else:
                # Check can fit this
                if self.traces.shape[0] <= self._numTraces:
                    if self._numTraces >= self.tracehint:
                        # Tracehint wrong - increase by 25
                        self.tracehint += 25

                    # Do a resize now to allocate more memory
                    self.traces.resize((self.tracehint, self.traces.shape[1]))

                #Validate traces fit - if too short warn & pad (prevents aborting long captures)
                pad = self.traces.shape[1] - len(trace)
                if pad > 0:
                    logging.warning('Trace too short (length=%d)' % len(trace) + " *This MAY SUGGEST DATA CORRUPTION*")
                    logging.warning('Padding with %d zero points' % pad)
                    trace = np.concatenate((trace, [0]*pad))

                self.traces[self._numTraces][:] = trace
        except MemoryError:
            raise Warning("Failed to allocate/resize array for %d x %d, if you have sufficient memory it may be fragmented. Use smaller segments and retry." % (self.tracehint, self.traces.shape[1]))
            
        self._numTraces += 1
        self.setDirty(True)
        self.writeDataToConfig()

    def setKnownKey(self, key):
        self.knownkey = key

    def addKey(self, key):
        self.keylist.append(key)

    def addTextin(self, data):
        self.textins.append(data)
        
    def addTextout(self, data):
        self.textouts.append(data)
        
    def getTrace(self, n):
        data = self.traces[n]

        #Following line will normalize all traces relative to each
        #other by mean & standard deviation
        #data = (data - np.mean(data)) / np.std(data)
        return data

    def getTextin(self, n):
        return self.textins[n]

    def getTextout(self, n):
        return self.textouts[n]

    def getKnownKey(self, n=0):
        if hasattr(self, 'keylist'):
            if self.keylist is not None:
                return self.keylist[n]

        return self.knownkey
    
    def getAuxDataConfig(self, newmodule):
        """
        Get auxilary data section in config file, searches based on both 'modname'
        and 'uniquedict'. Checks file itself & NOT the local database, since the
        auxilary data is not loaded into database.
        """

        # Get all section names
        for sname in list(self.config.config.keys()):
            # Find if starts with 'Aux Data'
            if sname.startswith("Aux Data"):

                # print "Found %s" % sname

                # Find if module name matches
                if sname.endswith(newmodule["sectionName"]):

                    # print "Found %s" % sname

                    # Finally confirm unique dictionary values
                    for k in list(newmodule["values"].keys()):
                        try:
                            if newmodule["values"][k]["definesunique"]:
                                try:
                                    if str(self.config.config[sname][k]) != str(newmodule["values"][k]["value"]):
                                        return None
                                except KeyError:
                                    return None
                        except KeyError:
                            pass

                    return self.config.config[sname]
        return None

    def addAuxDataConfig(self, newmodule):
        """Add a new module to the config file, place in aux data"""
        # newmodule is a dict of data

        # sectionName will be prepended with "Aux Data N - ", where NNNN
        # is a sequential number. The original section name will be called
        # .originalSectionName, which can be used when searching the system.
        # the NNNN will be also written as an integer to the .auxNumber member

        newdict = copy.deepcopy(newmodule)
        
        #Check dictionary
        maxNumber = 0
        for ad in self.config.attrList:
            if hasattr(ad, "auxNumber"):
                maxNumber = max(maxNumber, ad.auxNumber + 1)

        #Check configuration file
        for sname in list(self.config.config.keys()):
            # Find if starts with 'Aux Data'
            if sname.startswith("Aux Data"):
                maxNumber = max(int(re.findall(r'\d+', sname)[0]) + 1, maxNumber)

        newdict["auxNumber"] = maxNumber
        newdict["originalSectionName"] = newdict["sectionName"]
        newdict["sectionName"] = "Aux Data %04d - " % maxNumber + newdict["sectionName"]

        self.config.attrList.append(newdict)
        self.config.syncFile(sectionname=newdict["sectionName"])

        return newdict

    def prepareDisk(self):
        """Placeholder called after creating a new file setup, but before actually writing traces to it"""
        
        #if self.traces is None:
        #    self.traces = np.zeros((self.tracehint, self.pointhint))
        
    def loadAllConfig(self):
        """Placeholder for loading configuration data ONLY. May not be required."""
        pass
    
    def loadAllTraces(self, directory=None, prefix=""):
        """Placeholder for load command. May not actually read everything into memory depending on format."""
        self._isloaded = True
        raise AttributeError("%s doesn't have this method implemented"%self.__class__.__name__)

    def unloadAllTraces(self):
        """Placeholder for unload command. If loadAllTraces unloaded into memory, this may drop from memory. May not be implemented."""
        self._isloaded = False

    def saveAuxData(self, extraname, data):
        """Placeholder for command to save auxiliary data into some location which follows traces (e.g. same database/folder with same prefix.)"""
        raise AttributeError("%s doesn't have this method implemented" % self.__class__.__name__)
    
    def saveAllTraces(self, directory, prefix=""):
        """Placeholder for save command."""
        raise AttributeError("%s doesn't have this method implemented"%self.__class__.__name__)
    
    def copyTo(self, srcTraces=None, srcFormat=None):
        """Placeholder for copy/import command. Different from load as copies data INTO this classes format, possibly from another format"""
        raise AttributeError("%s doesn't have this method implemented"%self.__class__.__name__)
    
    def closeAll(self, clearTrace=True, clearText=True, clearKeys=True):
        """Writer is done, can close/save any files."""               
        #raise AttributeError("%s doesn't have this method implemented"%self.__class__.__name__)
        pass

    def validateSettings(self):
        """Check settings, log any messages to special setup window"""
        return []

    def isLoaded(self):
        """Returns true if you can use getTrace, getTextin, etc methods"""
        return self._isloaded

    def __repr__(self):
        return 'TraceContainerNative(number_of_traces={}, trace_length={})'.format(self.numTraces(), self.numPoints())
        
        
if __name__ == "__main__":
    test = TraceContainer()
    wave = [1,2,3,4,5,6]
    test.addTrace(wave, None, None, None)
    test.addTrace(wave, None, None, None)
    test.addTrace(wave, None, None, None)
    print(test.numTraces())
    print(test.numPoints())



#!/usr/bin/python
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_DEBUG
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
import logging
import pickle
import numpy as np
from ._base import TraceContainer
from ._cfgfile import makeAttrDict

try:
    import umysql as sql
except ImportError as e:
    # This isn't really needed, no need to bother users
    # print "umysql required: https://pypi.python.org/pypi/umysql"
    raise ImportError(e)


class TraceContainerMySQL(TraceContainer):
    _name = "MySQL"

    def __init__(self, openMode = False):
        super(TraceContainerMySQL, self).__init__()
        self.db = None
        self.idOffset = 0
        self.lastId = 0
        self.openMode = openMode
        self.fmt = None

        traceParams = [{'name':'MySQL Configuration', 'type':'group', 'children':[
                        {'name':'Server Address', 'key':'addr', 'type':'str', 'value':'127.0.0.1'},
                        {'name':'Server Port', 'key':'port', 'type':'int', 'value':'3306'},
                        {'name':'Username', 'key':'user', 'type':'str', 'value':'root'},
                        {'name':'Password', 'key':'password', 'type':'str', 'value':'admin'},
                        {'name':'Database', 'key':'database', 'type':'str', 'value':'CWTraces'},
                        {'name':'Table Name', 'key':'tableName', 'type':'str', 'value':'', 'readonly':True}
                      ]}]

        if self.openMode == False:
            traceParams[0]['children'].append({'name':'Table Naming', 'key':'tableNameType', 'type':'list', 'values':{'Auto-Prefix':'prefix'}, 'value':'prefix'})
        else:
            traceParams[0]['children'].append({'name':'Relist Tables', 'key':'tableListAct', 'type':'action'})
            traceParams[0]['children'].append({'name':'Table List', 'key':'tableNameList', 'type':'list', 'values':[], 'value':'', 'linked':['Table Name']})

        traceParams[0]['children'].append({'name':'Trace Format', 'key':'traceFormat', 'type':'list', 'values':['NumPy Pickle'], 'value':'NumPy Pickle', 'set':self.setFormat})
        self.params.addChildren(traceParams)

        #Connect actions if applicable
        try:
            self.getParams.findParam('tableListAct').opts['action'] = self.listAllTables
        except AttributeError:
            pass

        self.getParams.findParam('tableName').opts['get'] = self._getTableName

        #Save extra configuration options
        self.attrDict = makeAttrDict("MySQL Config", "mysql", self.getParams.traceParams)
        self.config.attrList.append(self.attrDict)

        #Format name must agree with names from TraceContainerFormatList
        self.config.setAttr("format", "mysql")

    def setFormat(self, fmt):
        self.fmt = fmt

    def format(self):
        if self.fmt is None:
            self.fmt = self.findParam('traceFormat').getValue()

        return self.fmt

    def makePrefix(self, mode='prefix'):
        if mode == 'prefix':
            prefix = self.config.attr('prefix')
            if prefix == "" or prefix is None:
                raise AttributeError("Prefix attribute not set in trace config")

            #Drop everything but underscore (_) for table name
            prefix = "tracedb_" + prefix
            prefix = prefix.replace("-","_")
            prefix = ''.join(c for c in prefix if c.isalnum() or c in("_"))
            return prefix

        raise ValueError("Invalid mode: %s"%mode)

    def prepareDisk(self):
        self.con()
        #CREATE DATABASE `cwtraces` /*!40100 COLLATE 'utf8_unicode_ci' */
        traceprefix = self.makePrefix(self.getParams.findParam('tableNameType').getValue())

        #Check version as simple validation
        result = self.db.query("SELECT VERSION()")
        logging.info('MySQL Version: %s' % result.rows[0][0])

        self.tableName = traceprefix

        self.db.query("CREATE TABLE IF NOT EXISTS %s(Id INT PRIMARY KEY AUTO_INCREMENT,\
         Textin VARCHAR(32),\
         EncKey VARCHAR(32),\
         Textout VARCHAR(32),\
         Wave MEDIUMBLOB)"%self.tableName)

    def con(self):
        if self.db is not None:
            self.db.close()

        db = sql.Connection()

        server = self.getParams.findParam('addr').getValue()
        port = int(self.getParams.findParam('port').getValue())
        user = self.getParams.findParam('user').getValue()
        password = self.getParams.findParam('password').getValue()
        database = self.getParams.findParam('database').getValue()

        #Connection
        db.connect(server,port,user,password,database)

        self.db = db

    def _getTableName(self):
        return self.getParams.findParam('tableNameList').getValue()

    def listAllTables(self):
        self.con()
        database = self.getParams.findParam('database').getValue()
        results = self.db.query("SHOW TABLES IN %s"%database)
        tables = []
        for r in results.rows:
            tables.append(r[0])
        self.getParams.findParam('tableNameList').setLimits(tables)


    def updatePointsTraces(self):
        res = self.db.query("SELECT COUNT(*) FROM %s" % self.tableName)
        self._numTraces = res.rows[0][0]

        wav = self.db.query("SELECT Wave FROM %s LIMIT 1 OFFSET %d" % (self.tableName, 0)).rows[0][0]
        self._numPoints = self.formatWave(wav, read=True).shape[0]

    def updateConfigData(self):
        self.con()

        self.tableName = self.getParams.findParam('tableName').getValue()
        res = self.db.query("SELECT COUNT(*) FROM %s" % self.tableName)
        self._numTraces = res.rows[0][0]
        self.config.setAttr('numTraces', self._numTraces)

        wav = self.db.query("SELECT Wave FROM %s LIMIT 1 OFFSET %d"%(self.tableName, 0)).rows[0][0]
        self._numPoints = self.formatWave(wav, read=True).shape[0]
        self.config.setAttr('numPoints', self._numPoints)

    def numTraces(self, update=False):
        if update:
            self.updateConfigData()
        return self._numTraces

    def numPoints(self, update=False):
        if update:
            self.updateConfigData()
        return self._numPoints

    def loadAllConfig(self):
        for p in self.getParams.traceParams[0]['children']:
            try:
                val = self.config.attr(p["key"], "mysql")
                self.getParams.findParam(p["key"]).setValue(val)
            except ValueError:
                pass
            #print "%s to %s=%s"%(p["key"], val, self.getParams.findParam(p["key"]).getValue())

    def loadAllTraces(self, path=None, prefix=None):
        self.updateConfigData()

    def formatWave(self, wave, read=False):
        if self.getParams.format() == "NumPy Pickle":
            if read == False:
                return pickle.dumps(wave, protocol=2)
            else:
                return np.array(pickle.loads(wave))
        else:
            raise AttributeError("Invalid Format for MySQL")

    def addTrace(self, trace, textin, textout, key, dtype=np.double):
        strTextin = ""
        for t in textin:
            strTextin += "%02X"%t

        strTextout = ""
        for t in textout:
            strTextout += "%02X" % t

        if key is not None:
            strKey = ""
            for t in key:
                strKey += "%02X"%t
        else:
            strKey = ""

        self.db.query("INSERT INTO %s(Textin, Textout, EncKey, Wave) VALUES('%s', '%s', '%s', "%(self.tableName, strTextin, strTextout,
                                                                                                strKey) + "%s)", (self.formatWave(trace),))
    def saveAll(self):
        #Save attributes from config settings
        for t in self.getParams.traceParams[0]['children']:
            self.config.setAttr(t["key"],  self.getParams.findParam(t["key"]).getValue() ,"mysql")

        #Save table name/prefix too
        self.config.setAttr("tableName", self.tableName, "mysql")
        self.config.saveTrace()

    def closeAll(self, clearTrace=True, clearText=True, clearKeys=True):
        # self.saveAllTraces(os.path.dirname(self.config.configFilename()), prefix=self.config.attr("prefix"))

        # Release memory associated with data in case this isn't deleted
        if clearTrace:
            self.traces = None

        if clearText:
            self.textins = None
            self.textouts = None

        if clearKeys:
            self.keylist = None
            self.knownkey = None

        if self.db is not None:
            self.db.close()
        self.db = None

    def getTrace(self, n):
        wv = self.db.query("SELECT Wave FROM %s LIMIT 1 OFFSET %d"%(self.tableName, n)).rows[0][0]
        return self.formatWave(wv, read=True)

    def asc2list(self, asc):
        lst = []
        for i in range(0,len(asc),2):
            lst.append( int(asc[i:(i+2)], 16) )
        return lst

    def getTextin(self, n):
        asc = self.db.query("SELECT Textin FROM %s LIMIT 1 OFFSET %d"%(self.tableName, n)).rows[0][0]
        return self.asc2list(asc)

    def getTextout(self, n):
        asc = self.db.query("SELECT Textout FROM %s LIMIT 1 OFFSET %d"%(self.tableName, n)).rows[0][0]
        return self.asc2list(asc)

    def getKnownKey(self, n=None):
        if n is None:
            n = 0
        asc = self.db.query("SELECT EncKey FROM %s LIMIT 1 OFFSET %d"%(self.tableName, n)).rows[0][0]
        return self.asc2list(asc)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._base import PreprocessingBase

class PassThrough(PreprocessingBase):
    """
    To-do list:
    [x] Nothing
    """
    _name = "Pass Through"

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self.findParam('Enabled').hide()

    def getTrace(self, n):
        return self._traceSource.getTrace(n)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

#
# This file performs forward AND backwards key scheduling. Can work from arbitrary
# key locations (i.e. first to last, last to first, etc.)
#
# Currently only support AES-128 and AES-256
#
from chipwhisperer.common.utils.util import camel_case_deprecated


def sbox(inp):
    s =  [0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67,
            0x2b, 0xfe, 0xd7, 0xab, 0x76, 0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59,
            0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0, 0xb7,
            0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1,
            0x71, 0xd8, 0x31, 0x15, 0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05,
            0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75, 0x09, 0x83,
            0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29,
            0xe3, 0x2f, 0x84, 0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b,
            0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf, 0xd0, 0xef, 0xaa,
            0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c,
            0x9f, 0xa8, 0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc,
            0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2, 0xcd, 0x0c, 0x13, 0xec,
            0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19,
            0x73, 0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee,
            0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb, 0xe0, 0x32, 0x3a, 0x0a, 0x49,
            0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
            0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4,
            0xea, 0x65, 0x7a, 0xae, 0x08, 0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6,
            0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a, 0x70,
            0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9,
            0x86, 0xc1, 0x1d, 0x9e, 0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e,
            0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf, 0x8c, 0xa1,
            0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0,
            0x54, 0xbb, 0x16]
    return s[inp]


def invsbox(inp):
    s = [0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3,
            0x9e, 0x81, 0xf3, 0xd7, 0xfb , 0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f,
            0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb , 0x54,
            0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b,
            0x42, 0xfa, 0xc3, 0x4e , 0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24,
            0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25 , 0x72, 0xf8,
            0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d,
            0x65, 0xb6, 0x92 , 0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda,
            0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84 , 0x90, 0xd8, 0xab,
            0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3,
            0x45, 0x06 , 0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1,
            0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b , 0x3a, 0x91, 0x11, 0x41,
            0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6,
            0x73 , 0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9,
            0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e , 0x47, 0xf1, 0x1a, 0x71, 0x1d,
            0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b ,
            0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0,
            0xfe, 0x78, 0xcd, 0x5a, 0xf4 , 0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07,
            0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f , 0x60,
            0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f,
            0x93, 0xc9, 0x9c, 0xef , 0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5,
            0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61 , 0x17, 0x2b,
            0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55,
            0x21, 0x0c, 0x7d]
    return s[inp]


rcon = [0x8d, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36,
            0x6c, 0xd8, 0xab, 0x4d, 0x9a, 0x2f, 0x5e, 0xbc, 0x63, 0xc6, 0x97,
            0x35, 0x6a, 0xd4, 0xb3, 0x7d, 0xfa, 0xef, 0xc5, 0x91, 0x39, 0x72,
            0xe4, 0xd3, 0xbd, 0x61, 0xc2, 0x9f, 0x25, 0x4a, 0x94, 0x33, 0x66,
            0xcc, 0x83, 0x1d, 0x3a, 0x74, 0xe8, 0xcb, 0x8d, 0x01, 0x02, 0x04,
            0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36, 0x6c, 0xd8, 0xab, 0x4d,
            0x9a, 0x2f, 0x5e, 0xbc, 0x63, 0xc6, 0x97, 0x35, 0x6a, 0xd4, 0xb3,
            0x7d, 0xfa, 0xef, 0xc5, 0x91, 0x39, 0x72, 0xe4, 0xd3, 0xbd, 0x61,
            0xc2, 0x9f, 0x25, 0x4a, 0x94, 0x33, 0x66, 0xcc, 0x83, 0x1d, 0x3a,
            0x74, 0xe8, 0xcb, 0x8d, 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40,
            0x80, 0x1b, 0x36, 0x6c, 0xd8, 0xab, 0x4d, 0x9a, 0x2f, 0x5e, 0xbc,
            0x63, 0xc6, 0x97, 0x35, 0x6a, 0xd4, 0xb3, 0x7d, 0xfa, 0xef, 0xc5,
            0x91, 0x39, 0x72, 0xe4, 0xd3, 0xbd, 0x61, 0xc2, 0x9f, 0x25, 0x4a,
            0x94, 0x33, 0x66, 0xcc, 0x83, 0x1d, 0x3a, 0x74, 0xe8, 0xcb, 0x8d,
            0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1b, 0x36, 0x6c,
            0xd8, 0xab, 0x4d, 0x9a, 0x2f, 0x5e, 0xbc, 0x63, 0xc6, 0x97, 0x35,
            0x6a, 0xd4, 0xb3, 0x7d, 0xfa, 0xef, 0xc5, 0x91, 0x39, 0x72, 0xe4,
            0xd3, 0xbd, 0x61, 0xc2, 0x9f, 0x25, 0x4a, 0x94, 0x33, 0x66, 0xcc,
            0x83, 0x1d, 0x3a, 0x74, 0xe8, 0xcb, 0x8d, 0x01, 0x02, 0x04, 0x08,
            0x10, 0x20, 0x40, 0x80, 0x1b, 0x36, 0x6c, 0xd8, 0xab, 0x4d, 0x9a,
            0x2f, 0x5e, 0xbc, 0x63, 0xc6, 0x97, 0x35, 0x6a, 0xd4, 0xb3, 0x7d,
            0xfa, 0xef, 0xc5, 0x91, 0x39, 0x72, 0xe4, 0xd3, 0xbd, 0x61, 0xc2,
            0x9f, 0x25, 0x4a, 0x94, 0x33, 0x66, 0xcc, 0x83, 0x1d, 0x3a, 0x74,
            0xe8, 0xcb ]


def g_func(inp, rcon):
    #Step 1: change order
    newlist = [inp[1], inp[2], inp[3], inp[0]]

    #Step 2: s-box
    newlist = [sbox(t) for t in newlist]

    #Step 3: apply rcon
    newlist[0] ^= rcon

    return newlist


def h_func(inp):
    #Step 1: s-box
    newlist = [sbox(t) for t in inp]

    return newlist


def xor(l1, l2):
    return [l1[i] ^ l2[i] for i in range(0, len(l1))]


def key_schedule_rounds(input_key, input_round, desired_round):
    """Use key schedule to determine key for different round.

    When dealing with AES-256, inputkey is 16 bytes and inputround
    indicates round that bytes 0...15 are from.

    Args:
        input_key (list): List of bytes of starting key, 16/32 bytes
        input_round (int): Starting round number (i.e. 0 = first)
        desired_round (int): Desired round number (i.e. 10 = last for 16-byte)

    Returns:
         list: Key at desired round number. Can go forward or backwards.
    """

    #Some constants
    n = len(input_key)
    if n == 16:
        pass
    elif n == 32:
        desiredfull = desired_round
        desired_round = int(desired_round / 2)

        #Special case for inputround of 13, needed for 'final' round...
        if input_round != 13:
            if input_round % 2 == 1:
                raise ValueError("Input round must be divisible by 2")
            input_round = int(input_round / 2)
        else:
            if input_round <= desiredfull:
                if desiredfull < 13:
                    raise ValueError("Round = 13 only permissible for reverse")

                if desiredfull == 13:
                    return input_key[0:16]
                else:
                    return input_key[16:32]

    else:
        raise ValueError("Invalid keylength: %d"%n)

    rnd = input_round
    state = list(input_key)

    #Check if we are going forward or backwards
    while rnd < desired_round:
        rnd += 1

        #Forward algorithm, thus need at least one round
        state[0:4] = xor(state[0:4], g_func(state[(n-4):n], rcon[rnd]))
        for i in range(4, n, 4):
            if n == 32 and i == 16:
                inp = h_func(state[(i-4):i])
            else:
                inp = state[(i - 4):i]
            state[i:(i+4)] = xor(state[i:(i+4)], inp)

    while rnd > desired_round:
        #For AES-256 final-round is 13 as that includes 32 bytes
        #of key. Convert to round 12 then continue as usual...
        if n == 32 and rnd == 13:
            inputrnd = int(12/2)
            rnd = inputrnd
            oldstate = list(state[16:32])
            state[16:32] = state[0:16]

            for i in range(12, 0, -4):
                state[i:(i+4)] = xor(oldstate[i:(i+4)], oldstate[(i-4):i])
            state[0:4] = xor(oldstate[0:4], g_func(state[(n - 4):n], rcon[7]))

        if rnd == desired_round:
            break

        # Reverse algorithm, thus need at least one round
        for i in range(n-4, 0, -4):
            if n == 32 and i == 16:
                inp = h_func(state[(i-4):i])
            else:
                inp = state[(i - 4):i]
            state[i:(i+4)] = xor(state[i:(i+4)], inp)
        state[0:4] = xor(state[0:4], g_func(state[(n - 4):n], rcon[rnd]))
        rnd -= 1

    #For AES-256, we use half the generated key at once...
    if n == 32:
        if desiredfull % 2:
            state = state[16:32]
        else:
            state = state[0:16]

    #Return answer
    return state

keyScheduleRounds = camel_case_deprecated(key_schedule_rounds)

def test():
    #Manual tests right now - need to automate this.

    ##### AES-128 Tests
    print("**********AES-128 Tests***************")

    ik = [0]*16
    for i in range(0, 11):
        result = keyScheduleRounds(ik, 0, i)
        print((" ".join(["%2x"%d for d in result])))
        ok = result

    # 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
    # 62 63 63 63 62 63 63 63 62 63 63 63 62 63 63 63
    # 9b 98 98 c9 f9 fb fb aa 9b 98 98 c9 f9 fb fb aa
    # 90 97 34 50 69 6c cf fa f2 f4 57 33 0b 0f ac 99
    # ee 06 da 7b 87 6a 15 81 75 9e 42 b2 7e 91 ee 2b
    # 7f 2e 2b 88 f8 44 3e 09 8d da 7c bb f3 4b 92 90
    # ec 61 4b 85 14 25 75 8c 99 ff 09 37 6a b4 9b a7
    # 21 75 17 87 35 50 62 0b ac af 6b 3c c6 1b f0 9b
    # 0e f9 03 33 3b a9 61 38 97 06 0a 04 51 1d fa 9f
    # b1 d4 d8 e2 8a 7d b9 da 1d 7b b3 de 4c 66 49 41
    #b4 ef 5b cb 3e 92 e2 11 23 e9 51 cf 6f 8f 18 8e

    print("")

    for i in range(0, 11):  # 10 Rounds
        result = keyScheduleRounds(ok, 10, i)
        print((" ".join(["%2x" % d for d in result])))

    ##### AES-256 Tests
    print("**********AES-256 Tests***************")

    ik = [0]*32
    for i in range(0, 15):  # 14 Rounds
        result = keyScheduleRounds(ik, 0, i)
        print((" ".join(["%02x"%d for d in result])))

    # 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
    # 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
    # 62 63 63 63 62 63 63 63 62 63 63 63 62 63 63 63
    # aa fb fb fb aa fb fb fb aa fb fb fb aa fb fb fb
    # 6f 6c 6c cf 0d 0f 0f ac 6f 6c 6c cf 0d 0f 0f ac
    # 7d 8d 8d 6a d7 76 76 91 7d 8d 8d 6a d7 76 76 91
    # 53 54 ed c1 5e 5b e2 6d 31 37 8e a2 3c 38 81 0e
    # 96 8a 81 c1 41 fc f7 50 3c 71 7a 3a eb 07 0c ab
    # 9e aa 8f 28 c0 f1 6d 45 f1 c6 e3 e7 cd fe 62 e9
    # 2b 31 2b df 6a cd dc 8f 56 bc a6 b5 bd bb aa 1e
    # 64 06 fd 52 a4 f7 90 17 55 31 73 f0 98 cf 11 19
    # 6d bb a9 0b 07 76 75 84 51 ca d3 31 ec 71 79 2f
    # e7 b0 e8 9c 43 47 78 8b 16 76 0b 7b 8e b9 1a 62
    # 74 ed 0b a1 73 9b 7e 25 22 51 ad 14 ce 20 d4 3b
    #10 f8 0a 17 53 bf 72 9c 45 c9 79 e7 cb 70 63 85

    print("")

    ik = [0x74 ,0xed ,0x0b ,0xa1 ,0x73 ,0x9b ,0x7e ,0x25 ,0x22 ,
          0x51 ,0xad ,0x14 ,0xce ,0x20 ,0xd4 ,0x3b ,0x10 ,0xf8 ,
          0x0a ,0x17 ,0x53 ,0xbf ,0x72 ,0x9c ,0x45 ,0xc9 ,0x79 ,
          0xe7 ,0xcb ,0x70 ,0x63, 0x85]

    for i in range(0, 14):
        result = keyScheduleRounds(ik, 13, i)
        print((" ".join(["%2x"%d for d in result])))

if __name__ == "__main__":
    test()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from chipwhisperer.capture.utils.SerialProtocols import CWCalcClkDiv as CalcClkDiv
from chipwhisperer.capture.utils.SerialProtocols import strToBits as strToBits
from chipwhisperer.common.utils.parameter import Parameter, Parameterized, setupSetParam

CODE_READ       = 0x80
CODE_WRITE      = 0xC0

ADDR_TRIGCLKDIV = 36
ADDR_TRIGIOPROG = 37


class CWAdvTrigger(object):
    def __init__(self, oa):
        super(CWAdvTrigger, self).__init__()
        self.oa = oa

    def setExtPin(self, line):
        return

    def setIOPattern(self, pattern, form='units', clkdiv=None, hackit=3):
        ''' Setup the IO trigger pattern.'''
        "Form can be 'units' or 'seconds'. If set to 'units' you"
        "must also set the clkdiv parameter properly"

        # Pattern format:
        # [state, low, high]
        # state = 1/0
        # low = time in seconds/units
        # high = time in seconds/units
        # Special high variables:
        # Set to 'now' means next state after low time passes
        # high = 'now'
        # Set to 'wait' means no upper limit, but wait for transition
        # of IO line before changing state
        # high = 'wait'

        if len(pattern) > 63:
            raise ValueError("pattern too long: Hardware supports max of 64 pattern points")

        addr = 0
        for p in pattern:
            state = p[0]
            low = p[1]
            high = p[2]
            # TODO: support seconds
            if form == 'seconds':
                raise NotImplementedError

            if high == 'now':
                high = 511

            if high == 'wait':
                high = 510

            # Addr = 0 has several samples taken at 'full speed' before clock divider
            # gets pushed in. The following seems to typically work to compensate, but
            # at high speeds this might be wrong.
            # TODO: This might require us to know oversample rate for properly doing this
            if addr == 0:
                low += hackit
                high += hackit

                if (low >= 510) or (high >= 510):
                    raise ValueError("low or high for addr=0 too large!")

            # print "%04d: %2d - %2d" % (addr, low, high)

            self.writeOnePattern(addr, state, low, high)
            addr = addr + 1

        # Set done marker
        self.writeOnePattern(addr, 1, 255, 511)

        if clkdiv:
            self.writeClockDiv(clkdiv)

    def writeClockDiv(self, clkdiv):
        clkdiv = int(clkdiv)
        d = bytearray()
        d.append(clkdiv & 0xff)
        d.append((clkdiv >> 8) & 0xff)
        d.append((clkdiv >> 16) & 0xff)
        self.oa.sendMessage(CODE_WRITE, ADDR_TRIGCLKDIV, d)

    def reset(self):
        resp = self.oa.sendMessage(CODE_READ, ADDR_TRIGCLKDIV, Validate=False, maxResp=3)
        # Assert Reset
        resp[2] = resp[2] | 0x80
        self.oa.sendMessage(CODE_WRITE, ADDR_TRIGCLKDIV, resp)

        # Deassert Reset
        resp[2] = resp[2] & 0x7F
        self.oa.sendMessage(CODE_WRITE, ADDR_TRIGCLKDIV, resp)

    def writeOnePattern(self, addr, state, low, high):
        d = bytearray()
        d.append(low)
        d.append(high & 0xff)
        d.append((0x01 & (high >> 8)) | (state << 1))
        d.append(addr)
        self.oa.sendMessage(CODE_WRITE, ADDR_TRIGIOPROG, d)

    def processBit(self, state, cnt, first=False, last=False, var=1, osRate=3):
        cnt = osRate * cnt
        low = cnt - var
        high = cnt + var

        if low < 1:
            low = 1

        if high > 509:
            high = 509

        if first:
            high = 'wait'

        if last:
            high = 'now'

        return [state, low, high]

    def bitsToPattern(self, bits, osRate=3, threshold=1):
        pattern = []
        lastbit = 2
        bitcnt = 0
        first = True
        for b in bits:
            if b == lastbit:
                bitcnt = bitcnt + 1
            else:
                if bitcnt > 0:
                    # print "%d %d"%(lastbit, bitcnt)
                    pattern.append(self.processBit(lastbit, bitcnt, first=first, var=threshold, osRate=osRate))
                lastbit = b
                bitcnt = 1

            first = False
        pattern.append(self.processBit(lastbit, bitcnt, last=True))
        return pattern

    def strToPattern(self, string, startbits=1, stopbits=1, parity='none'):
        totalpat = strToBits(string, startbits, stopbits, parity)
        return self.bitsToPattern(totalpat)


class ChipWhispererDigitalPattern(Parameterized):
    """
    Communicates and drives with the Digital Pattern Match module inside the FPGA. 
    """
    _name = 'Digital Pattern Trigger Module'
    def __init__(self, oa):
        self.cwAdv = CWAdvTrigger(oa)
        self.oa = oa

        self.getParams().addChildren([
             {'name':'Serial Settings', 'key':'ser', 'type':'group', 'children':[
                 {'name':'Baud', 'key':'baud', 'type':'int', 'limits':(100, 500000), 'value':38400, 'step':100, 'action':self.updateSampleRate},
                 {'name':'Start Bits', 'key':'startbits', 'type':'int', 'limits':(1, 10), 'value':1},
                 {'name':'Stop Bits', 'key':'stopbits', 'type':'int', 'limits':(1, 10), 'value':1},
                 {'name':'Parity', 'key':'parity', 'type':'list', 'values':['none', 'even'], 'value':'none'},
             ]},

             # TODO: Need to confirm oversample rate stuff
             {'name':'Oversample Rate', 'key':'osrate', 'type':'int', 'limits':(2, 5), 'value':3, 'action':self.updateSampleRate},
             {'name':'Calculated Clkdiv', 'key':'calcclkdiv', 'type':'int', 'value':0, 'readonly':True},
             {'name':'Calculated Error', 'key':'calcerror', 'type':'int', 'value':100, 'suffix':'%', 'readonly':True},
             {'name':'Trigger Character', 'key':'trigpatt', 'type':'str', 'value':'""', 'action': lambda p : self.setPattern(p.getValue())},
             {'name':'Binary Pattern', 'key':'binarypatt', 'type':'str', 'value':''},
             {'name':'Reset Module', 'type':'action', 'action':self.reset},

             {'name':'Advanced Settings', 'key':'adv', 'type':'group', 'children':[
                 {'name':'Threshold', 'key':'threshold', 'type':'int', 'value':1, 'limits':(1, 10), 'action':self.reset},
                 {'name':'Initial Bit Correction', 'key':'initialcorrect', 'type':'int', 'value':3, 'limits':(0, 10), 'action':self.reset},
             ]}
        ])

        self.updateSampleRate()

    def reset(self, ignored=None):
        # Reload pattern
        self.setPattern(self.findParam('trigpatt').getValue())

    def updateSampleRate(self, ignored=None):
        res = CalcClkDiv(self.oa.hwInfo.sysFrequency(), self.findParam(['ser', 'baud']).getValue() * self.findParam('osrate').getValue())
        self.findParam('calcclkdiv').setValue(res[0], ignoreReadonly=True)
        self.findParam('calcerror').setValue(res[1] * 100, ignoreReadonly=True)
        self.clkdiv = res[0]
        self.reset()

    def setPattern(self, patt):
        patt = eval(patt, {}, {})

        if len(patt) > 1:
            logging.warning('IO Pattern too large! Restricted.')
            self.findParam('trigpatt').setValue(patt[0])
            return

        # Convert to bits
        bitpattern = strToBits(patt, startbits=self.findParam(['ser', 'startbits']).getValue(),
                               stopbits=self.findParam(['ser', 'stopbits']).getValue(),
                               parity=self.findParam(['ser', 'parity']).getValue())

        # Convert to pattern & Download
        try:
            pat = self.cwAdv.bitsToPattern(bitpattern, osRate=self.findParam('osrate').getValue(),
                                                       threshold=self.findParam(['adv', 'threshold']).getValue())
            self.cwAdv.setIOPattern(pat, clkdiv=self.clkdiv, hackit=self.findParam(['adv', 'initialcorrect']).getValue())

            bitstr = ""
            for t in bitpattern: bitstr += "%d" % t

        except ValueError as s:
            bitstr = "Error: %s" % s

        self.findParam('binarypatt').setValue(bitstr)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._base import PreprocessingBase
from chipwhisperer.analyzer.utils.fasterdtw import fastdtw
from chipwhisperer.common.utils.parameter import setupSetParam

class ResyncDTW(PreprocessingBase):
    """Align traces using the Dynamic Time Warp algorithm. Doesn't play well
    with noisy traces, but can remove random per-trace delays and synchronize
    entire trace.
    """
    _name = "Resync: Dynamic Time Warp"
    _description = "Aligns traces to match a reference trace using the Fast Dynamic Time Warp algorithm."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._rtrace = 0
        self._radius = 3

        self.params.addChildren([
            {'name':'Ref Trace', 'key':'reftrace', 'type':'int', 'get':self._getRefTrace, 'set':self._setRefTrace},
            {'name':'Radius', 'key':'radius', 'type':'int', 'get':self._getRadius, 'set':self._setRadius}
        ])

    @setupSetParam("Ref Trace")
    def _setRefTrace(self, num):
        self._rtrace = num

    def _getRefTrace(self):
        return self._rtrace

    @property
    def ref_trace(self):
        """The trace being used as a reference.

        Setter raises TypeError unless value is an integer."""
        return self._getRefTrace()

    @ref_trace.setter
    def ref_trace(self, num):
        if not isinstance(num, int):
            raise TypeError("Expected int; got %s" % type(num), num)
        self._setRefTrace(num)

    @setupSetParam("Radius")
    def _setRadius(self, radius):
        self._radius = radius

    def _getRadius(self):
        return self._radius

    @property
    def radius(self):
        """The radius used in the DTW calculation.

        This is an integer value that roughly describes how much the DTW
        algorithm is allowed to backtrack in its search. High values take
        longer but are better at dealing with many smaller delays.
        """
        return self._getRadius()

    @radius.setter
    def radius(self, radius):
        if not isinstance(radius, int):
            raise TypeError("Expected int; got %s" % type(radius), radius)
        self._setRadius(radius)
   
    def getTrace(self, n):
        if not self.enabled:
            return self._traceSource.getTrace(n)

        trace = self._traceSource.getTrace(n)
        ref_trace = self._traceSource.getTrace(self._rtrace)
        if trace is None or ref_trace is None:
            return None

        aligned = self._alignTraces(ref_trace, trace)
        return aligned
        
    def _alignTraces(self, ref, trace):
        N = self._traceSource.numPoints()
        r = self._radius
        #try:
        dist, path = fastdtw(ref, trace, radius=r, dist=None)
        #except:
        #    return None
        px = [x for x, y in path]
        py = [y for x, y in path]
        n = [0] * N
        s = [0.0] * N
        for x, y in path:
            s[x] += trace[y]
            n[x] += 1

        ret = [s[i] / n[i] for i in range(N)]
        return ret


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from chipwhisperer.common.utils.tracesource import PassiveTraceObserver
from chipwhisperer.common.utils.parameter import setupSetParam
from chipwhisperer.common.utils.util import camel_case_deprecated

def enforceLimits(value, limits):
    if value < limits[0]:
        value = limits[0]
    elif value > limits[1]:
        value = limits[1]
    return value


class AttackBaseClass(PassiveTraceObserver):
    """Generic Attack Interface"""
    _name= 'Attack Settings'
    _algos = {}

    def __init__(self):
        PassiveTraceObserver.__init__(self)
        self._itNum = 0
        self.getParams().getChild("Input").hide()
        self._traceStart = 0
        self._iterations = 1
        self._tracePerAttack = 1
        self._reportingInterval = 10
        self._pointRange = (0,0)
        self._targetSubkeys = []
        self._project = None
        self.useAbs = True
        self.attack = None
        self.attackModel = None

        self.getParams().addChildren([
            {'name':'Attack Algorithm', 'type':'list',  'values':self._algos, 'get':self.getAlgorithm, 'set':self.setAlgorithm, 'action':self.updateScript, 'childmode': 'parent'}
        ])
        models = None
        self.getParams().addChildren([
            {'name':'Crypto Algorithm', 'type':'list', 'values':models, 'action':self.refreshByteList, 'childmode':'child'},
            {'name':'Points Range', 'key':'prange', 'type':'range', 'get':self.get_point_range, 'set':self.set_point_range, 'action':self.updateScript},
        ])

        self.getParams().addChildren([
            {'name':'Starting Trace', 'key':'strace', 'type':'int', 'get':self.get_trace_start, 'set':self.set_trace_start, 'action':self.updateScript},
            {'name':'Traces per Attack', 'key':'atraces', 'type':'int', 'limits':(1, 1E6), 'get':self.get_traces_per_attack, 'set':self.set_traces_per_attack, 'action':self.updateScript},
            {'name':'Iterations', 'key':'runs', 'type':'int', 'limits':(1, 1E6), 'get':self.get_iterations, 'set':self.set_iterations, 'action':self.updateScript},
            {'name':'Reporting Interval', 'key':'reportinterval', 'type':'int', 'get':self.get_reporting_interval, 'set':self.set_reporting_interval, 'action':self.updateScript},
        ])
        # self.getParams().init()

        self.setAlgorithm(self._analysisAlgorithm)
        self.refreshByteList()
        self.updateScript()
        if __debug__: logging.debug('Created: ' + str(self))

    def tracesUpdated(self):
        if self._analysisAlgorithm is not None:
            self._analysisAlgorithm.tracesUpdated(self._traceSource)
        self.updateTraceLimits()
        self.updateScript()

    def getAlgorithm(self):
        return self._analysisAlgorithm

    @setupSetParam('Attack Algorithm')
    def setAlgorithm(self, analysisAlgorithm):
        self._analysisAlgorithm = analysisAlgorithm
        self._analysisAlgorithm.tracesUpdated(self._traceSource)

        if hasattr(self._analysisAlgorithm, 'scriptsUpdated'):
            self._analysisAlgorithm.scriptsUpdated.connect(self.updateScript)
        if hasattr(self._analysisAlgorithm, 'runScriptFunction'):
            self._analysisAlgorithm.runScriptFunction.connect(self.runScriptFunction.emit)

    def set_leak_model(self, leakage_object):
        """Set the leak model to leakage_object"""
        self.attackModel = leakage_object

    setLeakModel = camel_case_deprecated(set_leak_model)

    def get_leak_model(self):
        """Get the leak model for the attack"""
        return self.attackModel

    leakModel = get_leak_model

    def set_analysis_algorithm(self, analysis_algorithm, leakage_object=None):
        """Sets the algorithm used for analyzing the trace data

        You probably want cpa_algorithms.Progressive

        Args:
            analysis_algorithm (AlgorithmsBase): Algorithm used for analyzing
                trace data. Only use cpa_algorithms.Progressive for now.
            leakage_object (ModelsBase, optional): Model used to get the
                leakage of the target (i.e. the sbox output). Needs to be set
                before the attack can happen
        """
        self.attack = analysis_algorithm()
        self.attack.setProject(self._project)
        self.attackModel = leakage_object

    setAnalysisAlgorithm = camel_case_deprecated(set_analysis_algorithm)
    def process_known_key(self, inpkey):

        """
        Passes known first-round key (if available, may pass None).
        Returns key under attack which should be highlighted in graph
        """
        return inpkey
    processKnownKey = camel_case_deprecated(process_known_key)

    def process_traces(self, callback=None):
        """ Run the attack!

        Args:
            callback (function(), optional): Function called every reporting
                interval. Not called if None. Defaults to None.

        Returns:
            Statistics object for the attack
        """
        progressBar = None

        self.attack.setModel(self.attackModel)
        self.attack.get_statistics().clear()
        self.attack.setReportingInterval(self.get_reporting_interval())
        self.attack.setTargetSubkeys(self.get_target_subkeys())
        self.attack.setStatsReadyCallback(callback)

        for itNum in range(self.get_iterations()):
            startingTrace = self.get_traces_per_attack() * itNum + self.get_trace_start()
            endingTrace = startingTrace + self.get_traces_per_attack() - 1

            # TODO:support start/end point different per byte
            self.attack.addTraces(self.getTraceSource(), (startingTrace, endingTrace), progressBar, pointRange=self.get_point_range(None))

        return self.attack.get_statistics()

    processTraces = camel_case_deprecated(process_traces)

    # for backwards compatability
    def processTracesNoGUI(self, callback=None, show_progress_bar=False):
        return self.processTraces(callback, show_progress_bar)

    def setProject(self, project):
        self._project = project

    def project(self):
        return self._project

    def get_trace_start(self):
        return self._traceStart

    getTraceStart = camel_case_deprecated(get_trace_start)
    @setupSetParam("Starting Trace")
    def set_trace_start(self, tnum):
        self._traceStart = tnum

    setTraceStart = camel_case_deprecated(set_trace_start)
    def get_iterations(self):
        return self._iterations

    getIterations = camel_case_deprecated(get_iterations)
    @setupSetParam("Iterations")
    def set_iterations(self, its):
        self._iterations = its

    setIterations = camel_case_deprecated(set_iterations)
    def get_traces_per_attack(self):
        return self._tracePerAttack

    getTracesPerAttack = camel_case_deprecated(get_traces_per_attack)
    @setupSetParam("Traces per Attack")
    def set_traces_per_attack(self, trace):
        if trace < 0:
            #Get maximum traces from source
            ts = self.getTraceSource()
            if ts is None:
                raise ValueError("traceSource not yet set in attack - set TraceSource first to use automatic getTracesPerAttack")
            trace = self.getTraceSource().num_traces()
        self._tracePerAttack = trace

    setTracesPerAttack = camel_case_deprecated(set_traces_per_attack)
    def get_reporting_interval(self):
        return self._reportingInterval

    getReportingInterval = camel_case_deprecated(get_reporting_interval)
    @setupSetParam("Reporting Interval")
    def set_reporting_interval(self, ri):
        self._reportingInterval = ri

    setReportingInterval = camel_case_deprecated(set_reporting_interval)
    def get_point_range(self, bnum=None):
        return self._pointRange

    getPointRange = camel_case_deprecated(get_point_range)
    @setupSetParam("Points Range")
    def set_point_range(self, rng):
        if rng[1] < 0:
            ts = self.getTraceSource()
            if ts is None:
                raise ValueError("traceSource not yet set in attack - set TraceSource first to use automatic setPointRange")
            rng = (rng[0], ts.num_points())
        self._pointRange = rng

    setPointRange = camel_case_deprecated(set_point_range)
    def known_key(self):
        """Get the known key via attack"""
        key = self.process_known_key(self.getTraceSource().get_known_key(self.get_trace_start()))
        if key is None:
            key = [None] * len(self.get_statistics().diffs)
        return key

    knownKey = camel_case_deprecated(known_key)
    def set_target_subkeys(self, blist):
        self._targetSubkeys = blist

    setTargetSubkeys = camel_case_deprecated(set_target_subkeys)
    def get_target_subkeys(self):
        return self._targetSubkeys

    getTargetSubkeys = camel_case_deprecated(get_target_subkeys)

    def __del__(self):
        if __debug__: logging.debug('Deleted: ' + str(self))

    def getAbsoluteMode(self):
        return self.useAbs

    def refreshByteList(self, _=None):
        pass

    def getEnabledSubkeys(self):
        return None

    def get_statistics(self):
        return self.attack.getStatistics()

    getStatistics = camel_case_deprecated(get_statistics)
    def updateScript(self, _=None):
        if self._traceSource is None:
            return

        # Add attack 'other' functions such as template generators etc
        runs = self.findParam('runs')
        atraces = self.findParam('atraces')
        strace = self.findParam('strace')
        ri = self.findParam('reportinterval')

        #print "runs = %d\natraces= %d\nstrace = %d\n"%(runs.value(), atraces.value(), strace.value())

        if (runs.getValue() * atraces.getValue() + strace.getValue()) > self._traceSource.num_traces() or atraces.getValue()<=0:
            solv = (self._traceSource.num_traces() - strace.getValue()) / runs.getValue()
            solv = int(solv)
            atraces.setValue(1, blockAction = True)
            atraces.setLimits((1, solv))
            atraces.setValue(solv, blockAction = True)
        else:
            atraces.setLimits((1, self._traceSource.num_traces()))

        pointrng = self.findParam('prange').getValue()


    def updateTraceLimits(self):
        if self._traceSource is None:
            return

        self.findParam('prange').setLimits((0, self._traceSource.num_points()-1))
        self.findParam('prange').setValue((0, self._traceSource.num_points()-1))


        strace = self.findParam('strace')
        self.findParam('runs').setValue(1)
        atrace = self.findParam('atraces')

        strace.setLimits((0, self._traceSource.num_traces()-1))
        atrace.setValue(1, blockAction=True)
        atrace.setLimits((1, self._traceSource.num_traces()))
        atrace.setValue(self._traceSource.num_traces(), blockAction=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._smartcard_gui import SmartCardGUICard
from ._base import TargetTemplate
from chipwhisperer.common.utils.parameter import setupSetParam


class SmartCard(TargetTemplate):
    _name = "Smart Card"

    def __init__(self):
        TargetTemplate.__init__(self)

        self.driver = None
        self.protocol = None

        self.params.addChildren([
            {'name':'Reader Hardware', 'type':'list', 'values':readers, 'get':self.getConnection, 'set':self.setConnection},
            {'name':'SmartCard Protocol', 'type':'list', 'values':protocols, 'get':self.getProtocol, 'set':self.setProtocol},
            {'name':'SmartCard Explorer Dialog', 'type':'action', 'action':lambda _: self.getScgui().show()}
        ])

    def __del__(self):
        self.close()

    def keyLen(self):
        """ Return key length in BYTES """
        return 16

    def getConnection(self):
        return self.driver

    @setupSetParam("Reader Hardware")
    def setConnection(self, con):
        self.driver = con        
        if self.driver is not None:
            self.params.append(self.driver.getParams())

    def getProtocol(self):
        return self.protocol

    @setupSetParam("SmartCard Protocol")
    def setProtocol(self, con):
        self.protocol = con
        if self.protocol is not None:
            self.params.append(self.protocol.getParams())
            self.protocol.setReaderHardware(self.driver)

    def _con(self, scope = None):
        self.driver.con(scope)
        self.driver.flush()
        self.protocol.setReaderHardware(self.driver)

    def close(self):
        if self.driver != None:
            self.driver.close()
            # self.driver = None
        return
        
    def init(self):
        if not self.protocol: raise Warning("No protocol selected.")
        self.protocol.init()

    def reinit(self):
        if not self.protocol: raise Warning("No protocol selected.")
        self.protocol.reinit()
      
    def loadEncryptionKey(self, key):
        self.protocol.loadEncryptionKey(key)
        self.key = key
      
    def loadInput(self, inputtext):
        self.protocol.loadInput(inputtext)
        self.input = inputtext

    def isDone(self):
        return self.protocol.isDone()

    def readOutput(self):        
        return self.protocol.readOutput()

    def go(self):
        self.protocol.go()

    def setModeEncrypt(self):
        pass
    
    def checkEncryptionKey(self, key):
        return key

    def getScgui(self):
        if not hasattr(self, 'scgui'):
            self.scgui = SmartCardGUICard(self)
        return self.scgui



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2015-2019, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.chipwhisperer.com . ChipWhisperer is a registered
# trademark of NewAE Technology Inc in the US & Europe.
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import time
from datetime import datetime
import os.path
from ._base import TargetTemplate
from chipwhisperer.hardware.naeusb.naeusb import NAEUSB
from chipwhisperer.hardware.naeusb.pll_cdce906 import PLLCDCE906
from chipwhisperer.hardware.naeusb.fpga import FPGA
from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.util import camel_case_deprecated


class CW305_USB(object):
    REQ_SYSCFG = 0x22
    REQ_VCCINT = 0x31
    SYSCFG_CLKOFF = 0x04
    SYSCFG_CLKON = 0x05
    SYSCFG_TOGGLE = 0x06
    VCCINT_XORKEY = 0xAE


class CW305(TargetTemplate):

    """CW305 target object.

    This class contains the public API for the CW305 hardware.
    To connect to the CW305, the easiest method is::

        import chipwhisperer as cw
        scope = cw.scope()
        target = cw.target(scope,
                targets.CW305, bsfile=<valid FPGA bitstream file>)

    Note that connecting to the CW305 includes programming the CW305 FPGA.
    For more help about CW305 settings, try help() on this CW305 submodule:

       * target.pll
    """


    _name = "ChipWhisperer CW305 (Artix-7)"

    def __init__(self):
        TargetTemplate.__init__(self)
        self._naeusb = NAEUSB()
        self.pll = PLLCDCE906(self._naeusb, ref_freq = 12.0E6)
        self.fpga = FPGA(self._naeusb)

        self.hw = None
        self.oa = None

        self._woffset = 0x400

        self._clksleeptime = 1
        self._clkusbautooff = True
        self.last_key = bytearray([0]*16)


    def fpga_write(self, addr, data):
        """Write to an address on the FPGA

        Args:
            addr (int): Address to write to
            data (list): Data to write to addr

        Raises:
            IOError: User attempted to write to a read-only location
        """
        if addr < self._woffset:
            raise IOError("Write to read-only location: 0x%04x"%addr)

        return self._naeusb.cmdWriteMem(addr, data)

    def fpga_read(self, addr, readlen):
        """Read from an address on the FPGA

        Args:
            addr (int): Address to read from
            readlen (int): Length of data to read

        Returns:
            Requested data as a list
        """
        if addr > self._woffset:
            logging.info('Read from write address, confirm this is not an error')

        data = self._naeusb.cmdReadMem(addr, readlen)
        return data

    def usb_clk_setenabled(self, status):
        """ Turn on or off the Data Clock to the FPGA """
        if status:
            self._naeusb.sendCtrl(CW305_USB.REQ_SYSCFG, CW305_USB.SYSCFG_CLKON)
        else:
            self._naeusb.sendCtrl(CW305_USB.REQ_SYSCFG, CW305_USB.SYSCFG_CLKOFF)

    def usb_trigger_toggle(self, _=None):
        """ Toggle the trigger line high then low """
        self._naeusb.sendCtrl(CW305_USB.REQ_SYSCFG, CW305_USB.SYSCFG_TOGGLE)

    def vccint_set(self, vccint=1.0):
        """ Set the VCC-INT for the FPGA """

        # print "vccint = " + str(vccint)

        if (vccint < 0.6) or (vccint > 1.15):
            raise ValueError("VCC-Int out of range 0.6V-1.1V")

        # Convert to mV
        vccint = int(vccint * 1000)
        vccsetting = [vccint & 0xff, (vccint >> 8) & 0xff, 0]

        # calculate checksum
        vccsetting[2] = vccsetting[0] ^ vccsetting[1] ^ CW305_USB.VCCINT_XORKEY

        self._naeusb.sendCtrl(CW305_USB.REQ_VCCINT, 0, vccsetting)

        resp = self._naeusb.readCtrl(CW305_USB.REQ_VCCINT, dlen=3)
        if resp[0] != 2:
            raise IOError("VCC-INT Write Error, response = %d" % resp[0])

    def vccint_get(self):
        """ Get the last set value for VCC-INT """

        resp = self._naeusb.readCtrl(CW305_USB.REQ_VCCINT, dlen=3)
        return float(resp[1] | (resp[2] << 8)) / 1000.0

    def _con(self, scope=None, bsfile=None, force=False):
        """Connect to CW305 board, and download bitstream.

        If the target has already been programmed it skips reprogramming
        unless forced.

        Args:
            scope (ScopeTemplate): An instance of a scope object.
            bsfile (path): The path to the bitstream file to program the FPGA with.
            force (bool): Whether or not to force reprogramming.
            force (bool): Whether or not to force reprogramming.
        """

        self._naeusb.con(idProduct=[0xC305])
        if self.fpga.isFPGAProgrammed() == False or force:
            if bsfile is None:
                print("No FPGA Bitstream file specified.")
            elif not os.path.isfile(bsfile):
                print(("FPGA Bitstream not configured or '%s' not a file." % str(bsfile)))
            else:
                from datetime import datetime
                starttime = datetime.now()
                status = self.fpga.FPGAProgram(open(bsfile, "rb"), exceptOnDoneFailure=False)
                stoptime = datetime.now()
                if status:
                    logging.info('FPGA Config OK, time: %s' % str(stoptime - starttime))
                else:
                    logging.warning('FPGA Done pin failed to go high, check bitstream is for target device.')
        self.usb_clk_setenabled(True)
        self.fpga_write(0x100+self._woffset, [0])
        self.pll.cdce906init()

    def _dis(self):
        if self._naeusb:
            self._naeusb.close()

    def checkEncryptionKey(self, key):
        """Validate encryption key"""
        return key

    def loadEncryptionKey(self, key):
        """Write encryption key to FPGA"""
        self.key = key
        key = key[::-1]
        self.fpga_write(0x100+self._woffset, key)

    def loadInput(self, inputtext):
        """Write input to FPGA"""
        self.input = inputtext
        text = inputtext[::-1]
        self.fpga_write(0x200+self._woffset, text)

    def is_done(self):
        """Check if FPGA is done"""
        result = self.fpga_read(0x50, 1)[0]

        if result == 0x00:
            return False
        else:
            # Clear trigger
            self.fpga_write(0x40+self._woffset, [0])
            # LED Off
            self.fpga_write(0x10+self._woffset, [0])
            return True

    isDone = camel_case_deprecated(is_done)

    def readOutput(self):
        """"Read output from FPGA"""
        data = self.fpga_read(0x200, 16)
        data = data[::-1]
        #self.newInputData.emit(util.list2hexstr(data))
        return data

    @property
    def clkusbautooff(self):
        """ If set, the USB clock is automatically disabled on capture.

        The USB clock is re-enabled after self.clksleeptime milliseconds.

        :Getter: Gets whether to turn off the USB clock on capture

        :Setter: Sets whether to turn off the USB clock on capture
        """
        return self._clkusbautooff

    @clkusbautooff.setter
    def clkusbautooff(self, state):
        self._clkusbautooff = state

    @property
    def clksleeptime(self):
        """ Time (in milliseconds) that the USB clock is disabled for upon
        capture, if self.clkusbautooff is set.
        """
        return self._clksleeptime

    @clksleeptime.setter
    def clksleeptime(self, value):
        self._clksleeptime = value

    def go(self):
        """Disable USB clock (if requested), perform encryption, re-enable clock"""
        if self.clkusbautooff:
            self.usb_clk_setenabled(False)

        #LED On
        self.fpga_write(0x10+self._woffset, [0x01])

        time.sleep(0.001)
        self.usb_trigger_toggle()
        # self.FPGAWrite(0x100, [1])
        # self.FPGAWrite(0x100, [0])

        if self.clkusbautooff:
            time.sleep(self.clksleeptime/1000.0)
            self.usb_clk_setenabled(True)

    def simpleserial_read(self, cmd, pay_len, end='\n', timeout=250, ack=True):
        """Read data from target

        Mimics simpleserial protocol of serial based targets

        Args:
            cmd (str): Command to ues. Only accepts 'r' for now.
            pay_len: Unused
            end: Unused
            timeout: Unused
            ack: Unused

        Returns: Value from Crypto output register

        .. versionadded:: 5.1
            Added simpleserial_read to CW305
        """
        if cmd == "r":
            return self.readOutput()
        else:
            raise ValueError("Unknown command {}".format(cmd))

    def simpleserial_write(self, cmd, data, end=None):
        """Write data to target.

        Mimics simpleserial protocol of serial based targets.

        Args:
            cmd (str): Command to use. Target supports 'p' (write plaintext),
                and 'k' (write key).
            data (bytearray): Data to write to target
            end: Unused

        Raises:
            ValueError: Unknown command

        .. versionadded:: 5.1
            Added simpleserial_write to CW305
        """
        if cmd == 'p':
            self.loadInput(data)
            self.go()
        elif cmd == 'k':
            self.loadEncryptionKey(data)
        else:
            raise ValueError("Unknown command {}".format(cmd))

    def set_key(self, key, ack=False, timeout=250):
        """Checks if key is different from the last one sent. If so, send it.

        Args:
            key (bytearray):  key to send
            ack: Unused
            timeout: Unused

        .. versionadded:: 5.1
            Added set_key to CW305
        """
        if self.last_key != key:
            self.last_key = key
            self.simpleserial_write('k', key)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2017, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.parameter import Parameterized

class SPIFlashTemplate(Parameterized):


    _name= 'SPI Flash Programmer'

    def __init__(self):
        self.connectStatus = util.Observable(False)
        self.getParams()

    def selectionChanged(self):
        pass

    def close(self):
        pass

    def con(self, scope=None):
        """Connect to target"""
        self.connectStatus.setValue(True)

    def dis(self):
        """Disconnect from target"""
        self.close()
        self.connectStatus.setValue(False)

    def spi_command(self, command_in, num_data_to_read=0):
        """Do a SPI command & return just the result"""
        padded_data_out = command_in + [0]*num_data_to_read
        response = self.hw_spi_write(padded_data_out)
        return list(response[len(command_in):])

    def hw_spi_write(self, data_to_send):
        """
        TX a certain number of bytes (in data_to_send) & also record state of input pin.

        Returns:
            list: Received data bytes over SPI
        """
        raise NotImplementedError




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._base import PreprocessingBase
import numpy as np
from chipwhisperer.common.utils.parameter import setupSetParam

        
class DecimationFixed(PreprocessingBase):
    """Throw out some points in each trace, keeping only one in N points."""
    _name = "Decimation: Fixed"
    _description = "Decimate (reduce the sample rate) by a fixed factor."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._dec_factor = 2
        self.params.addChildren([
            {'name':'Decimation = N:1', 'key':'decfactor', 'type':'int', 'get':self._getDecFactor, 'set':self._setDecFactor, 'limits':(1, 1000)}
        ])

    @setupSetParam("Decimation = N:1")
    def _setDecFactor(self, dec=1):
        self._dec_factor = dec

    def _getDecFactor(self):
        return self._dec_factor

    @property
    def dec_factor(self):
        """The current decimation factor. Only one in <dec_factor> samples are
        returned.

        Setter raises ValueError if new factor isn't an int"""
        return self._getDecFactor()

    @dec_factor.setter
    def dec_factor(self, dec):
        if not isinstance(dec, int):
            raise TypeError("Expected int; got %s" % type(dec), dec)
        self._setDecFactor(dec)

    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None

            outtrace = np.zeros(len(range(0, len(trace), self._dec_factor)))

            for idx, val in enumerate(range(0, len(trace), self._dec_factor)):
                outtrace[idx] = trace[val]

            return outtrace
        else:
            return self._traceSource.getTrace(n)

    def numPoints(self):
        if self.enabled:
            return len(range(0, self._traceSource.numPoints(), self._dec_factor))
        else:
            return self._traceSource.numPoints()

    def getSampleRate(self):
        return self._traceSource.getSampleRate()/self._dec_factor



"""Package containing side channel attacks

Attack Types:
CPA - Correlation Power Analysis Attack
Profiling - Profiling Power Analysis Attack

Algorithms:
Progressive - CPA via "online" calculation
"""

from .cpa import CPA
from .profiling import Profiling

